/******************************************************************************
*
* Copyright: Intellectual Property of Four Elements Capital Pte Ltd, Singapore.
* All rights reserved.
*
******************************************************************************/

package com.fourelementscapital.rfunction.common;

import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Vector;
import javax.servlet.http.HttpServletRequest;
import org.apache.commons.jcs.JCS;
import org.apache.commons.jcs.access.CacheAccess;
import org.apache.commons.jcs.engine.behavior.IElementAttributes;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import com.fourelementscapital.db.RFunctionDB;
import com.fourelementscapital.db.SchedulerDB;
import com.fourelementscapital.db.vo.ValueObject;


/**
 * This class exposes data to R function editor in Ajax call 
 * Remember all methods in the class are required user to be logged in
 */
@SuppressWarnings("unchecked")
public class RFunctionMgmt extends AbstractTeamOrgMgmt {
	
	private HttpServletRequest request=null;
	private Logger log = LogManager.getLogger(RFunctionMgmt.class.getName());
	private static CacheAccess<String, HashMap> lockcache=null;
	private  String cache_packaged_key="packaged_functions";	
	public static String RFUNCTION_FILE_EXTENSION=".r";
	
	private static String FUNCTION_ID="function_id";
	private static String LOCK_DURATION="duration";
	private static String USER="user";

	/**
	 * DWR invocation 
	 * @throws Exception
	 */
	public RFunctionMgmt() throws Exception {
		super();
	}
	
	/**
	 * JSP or internal invocation 
	 * @param request
	 * @throws Exception
	 */
	public RFunctionMgmt(HttpServletRequest request) throws Exception {
		super(request);
 
	}

	/**
	 * JCS caching for speed data retrival and not necessary to hit the database for every user.
	 * @return
	 * @throws Exception
	 */
	private static CacheAccess getLockCache() throws Exception {
		if(RFunctionMgmt.lockcache==null) RFunctionMgmt.lockcache=JCS.getInstance("lock-cache");
		return RFunctionMgmt.lockcache;
	}
	
	/**
	 * Locking r function while a user is editing 
	 * @param function_id
	 * @param seconds
	 * @return
	 * @throws Exception
	 */
	public boolean lockFunction(int function_id, long seconds) throws Exception {
		SchedulerDB sdb=SchedulerDB.getSchedulerDB();
		RFunctionDB rfdb=RFunctionDB .getRFunctionDB();
		try{
			sdb.connectDB();
			rfdb.connectDB();
			String usr=new SchedulerMgmt(getRequest()).getAuthorizedUser(sdb);
			if(seconds>0){			   			 
			   refreshCache(function_id, seconds,usr);		
			}else{
				rfdb.updateLock(function_id, usr);
			}
			return true;
		}catch(Exception e){
		
			throw e;
		}finally{
			sdb.closeDB();
			rfdb.closeDB();
		}
		
	}

	
	private void removeLockFromCache(int function_id, String usr ) throws Exception {
	       //String ky=usr+"_"+function_id;
		   String ky=usr+function_id;
		   if(getLockCache().get(ky)!=null)getLockCache().remove(ky);
	}

	private void refreshCache(int function_id, long seconds, String usr ) throws Exception {
		
	       //String ky=usr+"_"+function_id;
		   String ky=usr+function_id;
		   HashMap h=new HashMap();
		   h.put(RFunctionMgmt.FUNCTION_ID, function_id);
		   h.put(RFunctionMgmt.LOCK_DURATION, seconds);
		   h.put(RFunctionMgmt.USER,  usr);
		   
		   IElementAttributes att= getLockCache().getDefaultElementAttributes();
		   att.setMaxLife(seconds);
		   if(getLockCache().get(ky)!=null)getLockCache().remove(ky);
		   getLockCache().put(ky,h,att);
	}
	
	
	/**
	 * return data for folder generation.
	 * @return
	 * @throws Exception
	 */
	public Map getFolderTree() throws Exception {
		
		RFunctionDB rfdb=RFunctionDB.getRFunctionDB();
		
		try{
			HashMap rtn=new HashMap();
			
			rfdb.connectDB();			
			List glist=rfdb.listFunctionGroups();
			List folders=rfdb.listOfFolders();
			
			//TreeMap groups=new TreeMap();
			Vector groups=new Vector();
						
			for(Iterator it=glist.iterator();it.hasNext();){
				Map data=(Map)it.next();
                //groups.put(data.get("group_uid"),data.get("group_name"));
				
				ValueObject vo=new ValueObject();
				vo.setKey((String)data.get("group_uid"));
				vo.setValue((String)data.get("group_name"));
				groups.add(vo);
			}
			rtn.put("folders", folders);
			rtn.put("groups",groups);

			return rtn;
			
			
		}catch(Exception e){
			throw e;	
		}finally{
			
		}
	}	
}