/******************************************************************************
*
* Copyright: Intellectual Property of Four Elements Capital Pte Ltd, Singapore.
* All rights reserved.
*
******************************************************************************/

package com.fourelementscapital.auth;

import java.io.IOException;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * WikiAuthenticationServlet is a servlet to facilitate automatic Wiki authentication.
 * How to use this servlet, see documentation : http://wiki.4ecap.com/4ECwiki/Lib-auth#Automatic_Wiki_Authentication.
 */
public class WikiAuthenticationServlet extends HttpServlet {	
	
	private static final long serialVersionUID = 1L;
	
	private static final String LOGOUT = "logout";
	
	
	/**
	 * Handle post request, validate username & encrypted password by comparing them with Wiki database.
	 * If valid, generate session to be used throughout the app then redirect to app url.
	 * If not valid, redirect to wiki login url.
	 * @param request request
	 * @param response response
	 */	
	public void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {

		try {
			
			// get request parameters :
			String username = request.getParameter("username");
			String encryptedPassword = request.getParameter("encryptedPassword");
			String redirectUrl = request.getParameter("redirectUrl");
			
			// get wiki login url from config file 
			String wikiLoginUrl = Utils.GetConfigValue("wiki_login_url");

			// if there's already session then redirect to app url
			if (request.getSession().getAttribute(WikiAuthentication.SESSION_LOGGED_USER) != null) {
				response.sendRedirect(redirectUrl);
			}
			
			// if there's null parameter then redirect to wiki login
			else if (username == null || encryptedPassword == null || redirectUrl == null) {
				response.sendRedirect(wikiLoginUrl);
			}
			
			// validate username & encrypted password, if valid then redirect to app url
			else if (WikiAuthentication.validateUserEncryptedPassword(username, encryptedPassword)) {
				// set session for app using lib-auth
				request.getSession().setAttribute(WikiAuthentication.SESSION_LOGGED_USER, username);
				response.sendRedirect(redirectUrl);
			}
			
			// else, redirect to wiki login
			else {
				response.sendRedirect(wikiLoginUrl);				
			}
			
		}catch(Exception e){
			e.printStackTrace();
		}		
		
	}	
	
	
	/**
	 * Handle get request, logout (remove session) and redirect to a url.
	 * @param request request
	 * @param response response
	 */		
	public void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {

		String method = request.getParameter("method");
		
		// logout, remove session
		if(method != null && method.equals(this.LOGOUT)) {
			if (request.getSession().getAttribute(WikiAuthentication.SESSION_LOGGED_USER) != null) {
				request.getSession().removeAttribute(WikiAuthentication.SESSION_LOGGED_USER);
			}
		}

		// redirect to Wiki page
		response.sendRedirect(Utils.GetConfigValue("wiki_login_url"));		
	}


}

 
