/******************************************************************************
*
* Copyright: Intellectual Property of Four Elements Capital Pte Ltd, Singapore.
* All rights reserved.
*
******************************************************************************/

package com.fourelementscapital.db;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Vector;

import org.apache.commons.dbutils.BasicRowProcessor;



/**
 * This abstract class implements the methods that are related to Theme organisation tags
 */
public abstract class AbstractTeamOrgDB extends SuperDB{

	
	/**
	 * Get tag table  
	 * @return tag table
	 */	
	public abstract String getTagTable();  //tags
	
	/**
	 * Get tag item table  
	 * @return tag item table
	 */	
	public abstract String getTagItemTable(); //example: scheduler_tags
	
	/**
	 * Get tag item id  
	 * @return tag item id
	 */		
	public abstract String getTagItemId(); // example: scheduler_id or function_id;
	
	/**
	 * Get tag item follow table
	 * @return tag item follow table
	 */			
	public abstract String getTagItemFollowTable(); //example: scheduler_tags
	
	
	/**
	 * Returns list of All tags of the tool ( for example, if it was implemented for scheduler then it will return all scheduler themes)  
	 * @return tags
	 * @throws Exception
	 */
	public Vector getTags() throws Exception {
		 Vector rtn=new Vector();
		 PreparedStatement ps=this.connection().prepareStatement("select * FROM "+getTagTable()+" ORDER BY tagname");			
		 ResultSet rs=ps.executeQuery();
		 while(rs.next()){				
				rtn.add(new BasicRowProcessor().toMap(rs));
		 }
		 
			//added by rams on 5-june-2012
		rs.close();
		ps.close();			
		return rtn;
	 }
	 
	
	/**
	 * Get tags details
	 * @param tags tag names
	 * @return tags
	 * @throws Exception
	 */
	public Vector getTagsDetails(String tags[]) throws Exception {
		 Vector rtn=new Vector();
		 String stags=null;
		 for(int i=0;i<tags.length;i++){
			 stags=(stags==null)?"'"+tags[i]+"'":stags+",'"+tags[i]+"'";
		 }
		
		 if(stags!=null){
			 String query="select * FROM "+getTagTable()+" WHERE tagname IN ("+stags+") ORDER BY tagname";			 
			 PreparedStatement ps=this.connection().prepareStatement(query);			
			 ResultSet rs=ps.executeQuery();
			 while(rs.next()){
					rtn.add(new BasicRowProcessor().toMap(rs));
			 }
			//added by rams on 5-june-2012
			rs.close();
			ps.close();
		 }
		 return rtn;
	 }
	 
	 /**
	  * Get tag ids by item id
	  * @param item_id item id
	  * @return tag ids
	  * @throws Exception
	  */
	 public Vector getTagIds4Item(int item_id) throws Exception {
		 PreparedStatement ps=this.connection().prepareStatement("select tag_id FROM "+getTagItemTable()+" WHERE "+getTagItemId()+"=?");
		 ps.setInt(1, item_id);
		 ResultSet rs=ps.executeQuery();
		 Vector rtn=new Vector();
		 while(rs.next()){			 
				rtn.add(rs.getInt("tag_id"));
		 }
		//added by rams on 5-june-2012
		rs.close();
		ps.close();
			
		 return rtn;
	 }
	 
	 /**
	  * Get follow tag ids by item id
	  * @param item_id item id
	  * @return tag ids
	  * @throws Exception
	  */
	 public Vector getFollowTagIds4Item(int item_id) throws Exception {
		 PreparedStatement ps=this.connection().prepareStatement("select tag_id FROM "+getTagItemFollowTable()+" WHERE "+getTagItemId()+"=?");
		 ps.setInt(1, item_id);
		 ResultSet rs=ps.executeQuery();
		 Vector rtn=new Vector();
		 while(rs.next()){			 
				rtn.add(rs.getInt("tag_id"));
		 }
		//added by rams on 5-june-2012
		rs.close();
		ps.close();
			
		 return rtn;
	 }
	 
	 /**
	  * Update tag ids by item id
	  * @param item_id item id
	  * @param tag_ids tag ids
	  * @param removeExisting is remove existing
	  * @throws Exception
	  */
	 public void updateTagIds4Item(int item_id, Vector tag_ids,int removeExisting) throws Exception {
		 if(SchedulerDB.REMOVE_BEFORE_UPDATE==removeExisting){
			 PreparedStatement ps=this.connection().prepareStatement("DELETE FROM "+getTagItemTable()+" WHERE "+getTagItemId()+"=?");
			 ps.setInt(1, item_id);
			 ps.executeUpdate();
			 ps.close();
		 }
		 PreparedStatement ps1=this.connection().prepareStatement("INSERT INTO "+getTagItemTable()+"("+getTagItemId()+",tag_id) VALUES(?,?)");
		 for(Iterator i=tag_ids.iterator();i.hasNext();){			 
			 ps1.setInt(1, item_id);
			 ps1.setInt(2, Integer.parseInt((String)i.next()));
			 ps1.executeUpdate();
		 }

		 ps1.close();
	 }
	 
	 /**
	  * Remove tag ids by item id
	  * @param item_id item id
	  * @param tag_id tag id
	  * @throws Exception
	  */	 
	 public void removeTagIds4Item(int item_id, int tag_id) throws Exception {
		 PreparedStatement ps=this.connection().prepareStatement("DELETE FROM "+getTagItemTable()+" WHERE "+getTagItemId()+"=? and tag_id=?");
		 ps.setInt(1, item_id);
		 ps.setInt(2, tag_id);
		 ps.executeUpdate();
		 ps.close();
	 }
	 
	 /**
	  * Update tag item by tag id
	  * @param item_id item id
	  * @param tagids tag ids
	  * @throws Exception
	  */	 
	 public void updateItemTagIds(int item_id, List<Integer> tagids) throws Exception {
		  
		 String query="DELETE FROM "+getTagItemTable()+"  WHERE id IN (select * from (select a.id from "+getTagItemTable()+" as a left outer join "+getTagTable()+" as b on a.tag_id = b.id where "+getTagItemId()+"=? and left(b.tagname,3)<>'usr') temp_table ) ";
		 PreparedStatement ps=this.connection().prepareStatement(query);
		 ps.setInt(1, item_id);		 
		 ps.executeUpdate();
		 ps.close();
		 
		 
		 PreparedStatement ps1=this.connection().prepareStatement("INSERT INTO "+getTagItemTable()+"("+getTagItemId()+",tag_id) VALUES(?,?)");
		 for(Integer i:tagids){
			 ps1.setInt(1, item_id);
	     	ps1.setInt(2, i);
		 	ps1.executeUpdate();
		 }
		 ps1.close();
	 }
	 
	 /**
	  * Update follower tag id
	  * @param item_id item id
	  * @param tagids tag ids
	  * @throws Exception
	  */	 
	 public void updateFollwerTagIds(int item_id, List<Integer> tagids) throws Exception {
		  
		 String query="DELETE FROM "+getTagItemFollowTable()+"  WHERE id IN (select * from (select a.id from "+getTagItemFollowTable()+" as a left outer join "+getTagTable()+" as b on a.tag_id = b.id where "+getTagItemId()+"=? and left(b.tagname,3)<>'usr') temp_table ) ";
		 PreparedStatement ps=this.connection().prepareStatement(query);
		 ps.setInt(1, item_id);		 
		 ps.executeUpdate();
		 ps.close();
		 
		 
		 PreparedStatement ps1=this.connection().prepareStatement("INSERT INTO "+getTagItemFollowTable()+"("+getTagItemId()+",tag_id) VALUES(?,?)");
		 for(Integer i:tagids){
			 ps1.setInt(1, item_id);
	     	ps1.setInt(2, i);
		 	ps1.executeUpdate();
		 }
		 ps1.close();
	 }
	 
	 /**
	  * Get theme tags
	  * @param item_id item id
	  * @return theme tags
	  * @throws Exception
	  */	 
	 public List getThemeTags(int item_id) throws Exception {
		 
		 String q="select replace(b.tagname,'thm-','') as theme FROM "+getTagItemTable()+" as a ";
		 q+="left outer join "+getTagTable()+" as b on a.tag_id=b.id ";
		 q+="where a."+getTagItemId()+"=? ";
		 q+="and left(b.tagname,4)='thm-' ";

		 PreparedStatement ps=this.connection().prepareStatement(q);
		 ps.setInt(1, item_id);
		 
		 ResultSet rs=ps.executeQuery();
		 ArrayList rtn=new ArrayList();
		 while(rs.next()){			 
				rtn.add(rs.getString("theme"));
		 }
		 return rtn;
	 }
	 
	 /**
	  * Get theme tag name
	  * @param tag_id tag id
	  * @return theme tag name
	  * @throws Exception
	  */	 
	 public String getThemeTagName(int tag_id) throws Exception {
		 
		 String q="select replace(a.tagname,'thm-','') as theme FROM "+getTagTable()+" as a where id=?";	 
		 
 
		 PreparedStatement ps=this.connection().prepareStatement(q);
		 ps.setInt(1, tag_id);
		 
		 ResultSet rs=ps.executeQuery();
		 String rtn=null;
		 if(rs.next()){			 
				rtn= rs.getString("theme");
		 }
		 return rtn;
	 }

	 /**
	  * Get theme names by item id
	  * @param item_id item id
	  * @return theme names
	  * @throws Exception
	  */	 
	 public ArrayList getThemeNames4Item(int item_id) throws Exception {
		 PreparedStatement ps=this.connection().prepareStatement("select b.tagname FROM "+getTagItemTable()+" as a left outer join "+getTagTable()+" as b on a.tag_id=b.id WHERE a."+getTagItemId()+"=? and left(b.tagname,4)<>'usr-'");
		 ps.setInt(1, item_id);
		 ResultSet rs=ps.executeQuery();
		 ArrayList rtn=new ArrayList();
		 while(rs.next()){			 
			    String tagname=rs.getString("tagname");
			    tagname=tagname.replaceAll("thm-", "");
				rtn.add(tagname);
		 }
		//added by rams on 5-june-2012
		rs.close();
		ps.close();			 
		return rtn;
	 }
	 
	 
	 /**
	  * Get followers tag by item id
	  * @param item_id item id
	  * @return followers tag
	  * @throws Exception
	  */	 
	 public ArrayList getFollowTags4Item(int item_id) throws Exception {
		 PreparedStatement ps=this.connection().prepareStatement("select b.tagname FROM "+getTagItemFollowTable()+" as a left outer join "+getTagTable()+" as b on a.tag_id=b.id WHERE a."+getTagItemId()+"=? and left(b.tagname,4)<>'usr-'");
		 ps.setInt(1, item_id);
		 ResultSet rs=ps.executeQuery();
		 ArrayList rtn=new ArrayList();
		 while(rs.next()){			 
			    String tagname=rs.getString("tagname");
			    tagname=tagname.replaceAll("thm-", "");
				rtn.add(tagname);
		 }
		//added by rams on 5-june-2012
		rs.close();
		ps.close();			 
		return rtn;
	 }
	 
	 
	 /**
	  * Get SVN user for Wiki user
	  * @param w_user wiki user
	  * @return svn user
	  * @throws Exception
	  */		 
	 public Map getSVNUser4WikiUser(String w_user) throws Exception {
			
			PreparedStatement ps5=this.connection().prepareStatement("SELECT * FROM users where svn_username=?");
			
			ps5.setString(1, w_user);
			ResultSet rs5= ps5.executeQuery();
			Map rtn=new HashMap();
			if(rs5.next()){
				rtn=new BasicRowProcessor().toMap(rs5);
			}
			//added by rams on 5-june-2012
			rs5.close();
			ps5.close();
			return rtn;
		}

}

 
