/******************************************************************************
*
* Copyright: Intellectual Property of Four Elements Capital Pte Ltd, Singapore.
* All rights reserved.
*
******************************************************************************/

package com.fourelementscapital.db;

import java.util.List;
import java.util.Map;
import java.util.Vector;

import com.fourelementscapital.db.vo.FlexiField;

/**
 * Abstract class implements methods related to Flexi Field
 */
public abstract class FlexiFieldDB extends SuperDB {

	/**
	 * Connect database 
	 */
	public void connectDB() throws Exception {
		super.connectDB(DB_NAME_BBSYNC);
	}		

	/**
	 * Get flexi fields
	 * @param tablename tablename string
	 * @return flexi fields
	 * @throws Exception
	 */	
	public abstract List getFlexiFields(String tablename) throws Exception;

	/**
	 * Add flexi field
	 * @param ffield flexi field
	 * @throws Exception
	 */	
	public abstract void addFlexiField(FlexiField ffield) throws Exception;
	
	/**
	 * Update flexi field
	 * @param id flexi field id
	 * @param ffield flexi field
	 * @throws Exception
	 */		
	public abstract void updateFlexiField(long id,FlexiField ffield) throws Exception;

	/**
	 * Delete flexi field
	 * @param id flexi field id
	 * @throws Exception
	 */		
	public abstract void deleteFlexiField(long id) throws Exception;

	/**
	 * Get flexi field data
	 * @param tablename table name
	 * @param idfield id field
	 * @param idvalue id
	 * @return flexi field data
	 * @throws Exception
	 */			
	public abstract Map getFlexiFieldData (String tablename, String idfield, String idvalue) throws Exception;

	/**
	 * Get flexi field data for lucene token
	 * @param tablename table name
	 * @param idfield id field
	 * @param idvalue id
	 * @return flexi field data
	 * @throws Exception
	 */		
	public abstract Vector<Map> getFlexiFieldData4LuceneToken (String tablename, String idfield, String idvalue) throws Exception;
	
	/**
	 * Save flexi field data
	 * @param tablename table name
	 * @param idfield id field
	 * @param idvalue id
	 * @param data data
	 * @throws Exception
	 */			
	public abstract void saveFlexiFieldData (String tablename, String idfield, String idvalue, Map data) throws Exception;
	
	
	/**
	 * Update flexi field order
	 * @param id flexi field id
	 * @param order display order
	 * @throws Exception
	 */			
	public abstract void updateFlexiFieldOrder(long id,int order) throws Exception ;
	
}

 
