/******************************************************************************
*
* Copyright: Intellectual Property of Four Elements Capital Pte Ltd, Singapore.
* All rights reserved.
*
******************************************************************************/

package com.fourelementscapital.db;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Vector;

/**
 * Abstract class implements methods related to IExec
 */
public abstract class IExecDB extends SuperDB {
	
	/**
	 * Connect database 
	 */
	public void connectDB() throws Exception {
		super.connectDB(DB_NAME_IEXEC);
	}	

	/**
	 * List groups
	 * @return groups
	 * @throws Exception
	 */		
	public abstract List listGroups() throws Exception;
	
	/**
	 * Set group order
	 * @param groupids group id list
	 * @throws Exception
	 */			
	public abstract void setGroupOrder( Vector groupids) throws Exception;
	
	/**
	 * List folders
	 * @return ie folder list
	 * @throws Exception
	 */		
	public abstract List listFolders( ) throws Exception;	
	
	/**
	 * List strategies
	 * @return strategies
	 * @throws Exception
	 */		
	public abstract List listStrategies() throws Exception;	
	
	/**
	 * Get strategy
	 * @param strategy_id strategy id
	 * @return strategy
	 * @throws Exception
	 */		
	public abstract Map getStrategy(int strategy_id) throws Exception;
	
	/**
	 * Check whether it is parent
	 * @param strategy_name strategy name
	 * @return true if it is parent strategy
	 * @throws Exception
	 */		
	public abstract boolean isParent(String strategy_name) throws Exception;
	
	/**
	 * Get strategies by names
	 * @param strategy_names strategy names
	 * @return strategies
	 * @throws Exception
	 */		
	public abstract Vector getStrategies(Vector strategy_names) throws Exception;
	
	/**
	 * Get folder name
	 * @param folder_id folder id
	 * @return folder name list
	 * @throws Exception
	 */			
	public abstract String getFolderName(int folder_id) throws Exception;
	
	/**
	 * Get unique contracts
	 * @param strategy_name strategy name
	 * @return contracts
	 * @throws Exception
	 */		
	public abstract Vector getUniqueContracts(String strategy_name) throws Exception;
	
	/**
	 * Add parameters
	 * @param data parameter data
	 * @param strategy_name strategy name
	 * @throws Exception
	 */		
	public abstract void addParameters(ArrayList<Map> data,String strategy_name) throws Exception;
	
	/**
	 * Get parameter values
	 * @param strategy_name strategy name
	 * @param contract contract
	 * @return parameter values
	 * @throws Exception
	 */		
	public abstract Map getParameterValues(String strategy_name, String contract) throws Exception;
	
	/**
	 * Remove contract
	 * @param strategy_name strategy name
	 * @param contract contract
	 * @throws Exception
	 */			
	public abstract void removeContract(String strategy_name, String contract) throws Exception;
	
	/**
	 * Get strategy by name
	 * @param strategy_name strategy name
	 * @return strategy
	 * @throws Exception
	 */	
	public abstract Map getStrategy(String strategy_name) throws Exception;
	
	/**
	 * Create strategy
	 * @param folder_id folder id
	 * @param strategy_name strategy name
	 * @param path file name path
	 * @return strategy id
	 * @throws Exception
	 */		
	public abstract int createStrategy(int folder_id,String strategy_name, String path) throws Exception;
	
	/**
	 * Create child strategy
	 * @param parent_name parent name
	 * @param strategy_name strategy name
	 * @return strategy id
	 * @throws Exception
	 */		
	public abstract int createChildStrategy(String parent_name,String strategy_name) throws Exception;
	
	/**
	 * Get folder id
	 * @param folder_name folder name
	 * @return folder id
	 * @throws Exception
	 */		
	public abstract int getFolderID(String folder_name) throws Exception;
	
	/**
	 * Create folder
	 * @param folder folder name
	 * @param new_group_id new group id
	 * @return folder id
	 * @throws Exception
	 */		
	public abstract int createFolder(String folder, String new_group_id) throws Exception;
	
	/**
	 * List of folders
	 * @param group_id group id
	 * @return folders
	 * @throws Exception
	 */		
	public abstract List listOfFolders(String group_id) throws Exception;
	
	/**
	 * Update strategy folder
	 * @param strategy_id strategy id
	 * @param new_folder_id new folder id
	 * @throws Exception
	 */		
	public abstract void updateStrategyFolder(int strategy_id,int new_folder_id) throws Exception;
	
	/**
	 * Move folder
	 * @param folder_id folder id
	 * @param new_group_id new group id
	 * @throws Exception
	 */		
	public abstract void moveFolder(int folder_id, String new_group_id) throws Exception;
	
	/**
	 * Rename strategy
	 * @param strategy_id strategy id
	 * @param strategy_name strategy name
	 * @param file_name file name
	 * @throws Exception
	 */		
	public abstract void renameStrategy(int strategy_id, String strategy_name, String file_name) throws Exception;
	
	/**
	 * Rename parent strategy
	 * @param strategy_id strategy id
	 * @param strategy_name strategy name
	 * @param file_name file name
	 * @param old_strategy_name old strategy name
	 * @throws Exception
	 */		
	public abstract void renameParentStrategy(int strategy_id, String strategy_name, String file_name, String old_strategy_name) throws Exception;
	
	/**
	 * Get contract tree
	 * @return contract tree
	 * @throws Exception
	 */		
	public abstract Vector getContractTree() throws Exception;

	/**
	 * Get commodity tree
	 * @return commodity tree
	 * @throws Exception
	 */		
	public abstract Vector<String> getCommodityTree() throws Exception;
	
}

 
