/******************************************************************************
*
* Copyright: Intellectual Property of Four Elements Capital Pte Ltd, Singapore.
* All rights reserved.
*
******************************************************************************/

package com.fourelementscapital.db;

import java.util.List;
import java.util.Map;

/**
 * Abstract class implements methods related to Infrastructure
 */
public abstract class InfrastructureDB extends SuperDB { 
	
	public static final String APPLICATION_IEXEC = "iExec";
	public static final String APPLICATION_SCHEDULER_PEER_ASSOCIATION = "Scheduler.PeerAssociation";
	
	/**
	 * Connect database 
	 */
	public void connectDB() throws Exception {
		super.connectDB(DB_NAME_INFRASTRUCTURE);
	}
	
	/**
	 * Get themes by user
	 * @param user user
	 * @return themes
	 * @throws Exception
	 */	 
	public abstract Map getThemes4Users(String user) throws Exception;
	
	/**
	 * Get team organization
	 * @param themes themes
	 * @param hierarchy hierarchy
	 * @return team organization
	 * @throws Exception
	 */	 
	public abstract Map getTeamOrg(String themes, List hierarchy) throws Exception;

	/**
	 * Update Bloomberg daily counter
	 * @param date date
	 * @param count BBG daily counter
	 * @throws Exception
	 */
	public abstract void updateBloomberDailyCounter(String date, int count) throws Exception;
	
	/**
	 * get themes by username access 'X','B','C'
	 * @param username username
	 * @return themes
	 * @throws Exception
	 */
	public abstract List<String> getThemeByUsernameAccess(String username) throws Exception;
	
	/**
	 * Get access by app and username
	 * @param application application
	 * @param username username
	 * @return application user access
	 * @throws Exception
	 */	
	public abstract List getAccessByAppAndUsername(String application, String username) throws Exception;	
	
}


 