/******************************************************************************
*
* Copyright: Intellectual Property of Four Elements Capital Pte Ltd, Singapore.
* All rights reserved.
*
******************************************************************************/

package com.fourelementscapital.db;

import java.util.Map;
import java.util.Vector;

import com.fourelementscapital.db.vo.CommodityInfo;

/**
 * Abstract class implements methods related to reference
 */
public abstract class ReferenceDB extends SuperDB {
	
	/**
	 * Connect database 
	 */
	public void connectDB() throws Exception {
		super.connectDB(DB_NAME_BBSYNC);
	}	
	
	/**
	 * Get commodity contracts
	 * @return commodity contracts
	 * @throws Exception
	 */		 
	public abstract Map getComContracts() throws Exception;
	
	/**
	 * Delete field mapping
	 * @param id field mapping id
	 * @throws Exception
	 */			
	public abstract void deleteFieldMapping(int id) throws Exception;
	
	/**
	 * Add field mapping
	 * @param dbfield friendly name
	 * @param bbfield bb field name
	 * @throws Exception
	 */		
	public abstract void addFieldMapping(String dbfield, String bbfield) throws Exception;
	
	/**
	 * Get field mapping
	 * @return field mapping
	 * @throws Exception
	 */		
	public abstract Vector getFieldMapping() throws Exception;

	/**
	 * Delete security field mapping
	 * @param id security field mapping id
	 * @throws Exception
	 */	
	public abstract void deleteSecFieldMapping(int id) throws Exception;
	
	/**
	 * Add security field mapping
	 * @param dbfield friendly name
	 * @param bbfield bb field name
	 * @throws Exception
	 */		
	public abstract void addSecFieldMapping(String dbfield, String bbfield) throws Exception;
	
	/**
	 * Get security field mapping
	 * @return security field mapping
	 * @throws Exception
	 */		
	public abstract Vector getSecFieldMapping() throws Exception;
	
	/**
	 * Update commodity
	 * @param comm commodity info
	 * @throws Exception
	 */		
	public abstract void updateCommodity(CommodityInfo comm) throws Exception;

	/**
	 * Get commodity info
	 * @param commodity commodity
	 * @return commodity info
	 * @throws Exception
	 */		
	public abstract Map getCommodityInfo(String commodity) throws Exception;
	
	/**
	 * Get contract info
	 * @param contract contract name
	 * @return contract info
	 * @throws Exception
	 */		
	public abstract Map getContractInfo(String contract) throws Exception;

	/**
	 * Get security info
	 * @param security security name
	 * @return security info
	 * @throws Exception
	 */		
	public abstract Map getSecurityInfo(String security) throws Exception;
	
	/**
	 * Get contract BB info, this method may not be useful in future.
	 * @param contract contract
	 * @return contract BB info
	 * @throws Exception
	 */
	public abstract Vector getContractBBInfo(String contract) throws Exception;
	

	/**
	 * Get security BB info, this method may not be usefull in future.
	 * @param security security
	 * @return security BB info
	 * @throws Exception
	 */
	public abstract Vector getSecurityBBInfo(String security) throws Exception;
	

	/**
	 * Get all security field values
	 * @param friendlyname security field mapping friendly name
	 * @return security field values
	 * @throws Exception
	 */		
	public abstract Map getAllSecFieldValues(String friendlyname) throws Exception;

}

 
