/******************************************************************************
*
* Copyright: Intellectual Property of Four Elements Capital Pte Ltd, Singapore.
* All rights reserved.
*
******************************************************************************/

package com.fourelementscapital.db.mariadb;

import java.sql.Date;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.Statement;
import java.sql.Timestamp;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.Map;
import java.util.StringTokenizer;
import java.util.Vector;

import org.apache.logging.log4j.Logger;
import org.apache.logging.log4j.LogManager;

import com.fourelementscapital.db.BBSyncDB;
import com.fourelementscapital.db.GeneralUtilDB;
import com.fourelementscapital.db.vo.BBSyncTrigger;


/**
 * BBSync MariaDB DAO implementation
 */	
public class BBSyncDBMariaDB extends BBSyncDB {
private Logger log=LogManager.getLogger(BBSyncDB.class.getName());
	

	/**
	 * {@inheritDoc}
	 */	
	public Vector listAll() throws Exception {
		
		Statement st=this.connection().createStatement();
		ResultSet rs=st.executeQuery("Select * from bbsync order by name");
		Vector rtn=new Vector();
		while(rs.next()){
			//Map record=new BasicRowProcessor().toMap(rs);
			Map record = GeneralUtilDB.resultsetToMap(rs);
			int id=(Integer)record.get("id");
			record.put("fieldsids",getFieldIds(id));
			record.put("tickers", getContracts(id));
			rtn.add(record);
			
		}
		return rtn;
	}
	
	
	/**
	 * {@inheritDoc}
	 */	
	public ArrayList fieldMapping4BBSync(int bbsyncid) throws Exception {
		
		PreparedStatement ps=this.connection().prepareStatement("select * from field_mapping where id in (select field_id from bbsync_fields where bbsync_id=?)");		
		ps.setLong(1, bbsyncid);		
		ResultSet rs=ps.executeQuery();
		ArrayList rtn=new ArrayList();
		while(rs.next()){
			//Map record=new BasicRowProcessor().toMap(rs);
			Map record = GeneralUtilDB.resultsetToMap(rs);
			rtn.add(record);
			
		}
		rs.close();
		ps.close();
		return rtn;
	}
	
	
	/**
	 * {@inheritDoc}
	 */	
	public Map getDownloadQuery(long id) throws Exception {
		
		PreparedStatement ps=this.connection().prepareStatement("Select * from bbsync where id=?");
		ps.setLong(1, id);		
		ResultSet rs=ps.executeQuery();
		Map record=null;
		if(rs.next()){			
			//record=new BasicRowProcessor().toMap(rs);
			record = GeneralUtilDB.resultsetToMap(rs);
		}		
		rs.close();
		ps.close();
		if(record!=null){
			record.put("fieldsids",getFieldIds(id));
			record.put("tickers", getContracts(id));
		}
		return record;
	}

	
	/**
	 * Get field id
	 * @param bbsyncid bbsync id
	 * @return field ids
	 * @throws Exception
	 */	
	private Vector getFieldIds(long bbsyncid) throws Exception {
		PreparedStatement ps1=this.connection().prepareStatement("Select field_id from bbsync_fields where bbsync_id=?");
		ps1.setLong(1, bbsyncid);
		ResultSet rs1=ps1.executeQuery();
		Vector v=new Vector();
		while(rs1.next()){
			v.add((Integer)rs1.getInt("field_id")+""); //convert to string
		}
		ps1.close();
		rs1.clearWarnings();
		return v;

	}
	
	
	/**
	 * Get contracts
	 * @param bbsyncid bbsync id
	 * @return contract
	 * @throws Exception
	 */		
	private String getContracts(long bbsyncid) throws Exception {
		PreparedStatement ps1=this.connection().prepareStatement("Select contract from bbsync_contracts where bbsync_id=?");
		ps1.setLong(1, bbsyncid);
		ResultSet rs1=ps1.executeQuery();
		String tickers="";
		while(rs1.next()){
			tickers+=(tickers.equals(""))?rs1.getString("contract"):","+rs1.getString("contract"); 
		}
		ps1.close();
		rs1.clearWarnings();
		return tickers;

	}

	
	/**
	 * {@inheritDoc}
	 */	
	public Vector getFieldMapping() throws Exception {
		
		PreparedStatement ps=this.connection().prepareStatement("Select * from field_mapping");
		
		ResultSet rs=ps.executeQuery();
		Vector rtn=new Vector();
		while(rs.next()){			
			//rtn.add(new BasicRowProcessor().toMap(rs));
			rtn.add(GeneralUtilDB.resultsetToMap(rs));
		}
		return rtn;
	}
	 
	
	/**
	 * {@inheritDoc}
	 */	
	public void updateTriggeredDate(int id, Timestamp start, Timestamp end) throws Exception {
		PreparedStatement ps=this.connection().prepareStatement("UPDATE bbsync SET trigger_startedat=?, trigger_endedat=?,last_executed=? WHERE id=?");
		ps.setTimestamp(1, start);
		ps.setTimestamp(2, end);
		ps.setTimestamp(3, end);
		ps.setInt(4, id);
		ps.executeUpdate();		
		ps.close();
	}
	
	
	/**
	 * {@inheritDoc}
	 */	
	public void deleteFieldMapping(int id) throws Exception {
		PreparedStatement ps=this.connection().prepareStatement("DELETE FROM field_mapping where id=?");
		ps.setInt(1, id);
		ps.executeUpdate();
		ps.close();
	}
	
	
	/**
	 * {@inheritDoc}
	 */	
	public void addFieldMapping(String dbfield, String bbfield) throws Exception {
		PreparedStatement ps=this.connection().prepareStatement("INSERT INTO field_mapping(db_field, bb_field) VALUES (?,?)");
		ps.setString(1, dbfield);
		ps.setString(2, bbfield);
		ps.executeUpdate();
		ps.close();
	}


	/**
	 * {@inheritDoc}
	 */	
	public void updateContractLogs(Vector<String> contract,String marketsector,Timestamp lastsync, Map<String,String> scommodities, Collection fields) throws Exception {
		
		for(Iterator it=fields.iterator();it.hasNext();){
			String field=(String)it.next();
			PreparedStatement ps3=this.connection().prepareStatement("SELECT * FROM reference.contractSync WHERE contract_name=? AND contract_field=?");		
			PreparedStatement ps4=this.connection().prepareStatement("INSERT INTO reference.contractSync(last_sync,contract_name,contract_field) VALUES(?,?,?)");
			PreparedStatement ps5=this.connection().prepareStatement("UPDATE reference.contractSync SET last_sync=? WHERE contract_name=? AND contract_field=?");
			for(Iterator<String> i=contract.iterator();i.hasNext();){
				String cont=i.next();
				cont=cont.trim()+" "+marketsector;				
				ps3.setString(1, cont);
				ps3.setString(2, field);
				ResultSet rs=ps3.executeQuery();
				if(rs.next()){
					ps5.setTimestamp(1, lastsync);
					ps5.setString(2, cont.trim());
					ps5.setString(3, field);
					ps5.execute();				
				}else{
					ps4.setTimestamp(1, lastsync);
					ps4.setString(2, cont.trim());
					ps4.setString(3, field);
					ps4.execute();				
				}
				rs.close();
			}
			ps3.close();
			ps4.close();
			ps5.close();
				
		}
	}

	
	/**
	 * {@inheritDoc}
	 */	
	public void updateSecuritiesLogs(Vector<String> contract,String marketsector,Timestamp lastsync,Collection fields) throws Exception {
		
		for(Iterator it=fields.iterator();it.hasNext();){
			String field=(String)it.next();
			PreparedStatement ps3=this.connection().prepareStatement("SELECT * FROM reference.SecuritySync WHERE security_name=? AND security_field=?");		
			PreparedStatement ps4=this.connection().prepareStatement("INSERT INTO reference.SecuritySync(last_sync,security_name,security_field) VALUES(?,?,?)");
			PreparedStatement ps5=this.connection().prepareStatement("UPDATE reference.SecuritySync SET last_sync=? WHERE security_name=? AND security_field=?");
			for(Iterator<String> i=contract.iterator();i.hasNext();){
				String cont=i.next();
				cont=cont.trim()+" "+marketsector;				
				ps3.setString(1, cont);				
				ps3.setString(2, field);
				ResultSet rs=ps3.executeQuery();
				if(rs.next()){
					ps5.setTimestamp(1, lastsync);
					ps5.setString(2, cont.trim());					
					ps5.setString(3, field);
					ps5.execute();				
				}else{
					ps4.setTimestamp(1, lastsync);
					ps4.setString(2, cont.trim());
					ps4.setString(3, field);
					ps4.execute();				
				}
				rs.close();
			}
			ps3.close();
			ps4.close();
			ps5.close();
				
		}
		
	}
	

	/**
	 * {@inheritDoc}
	 */	
	public Vector<String> getContractNames2RefSync(boolean pendingonly) throws Exception {
		Statement st=this.connection().createStatement();
		ResultSet rs=null;
		if(pendingonly){
			rs=st.executeQuery("select * from contract_info WHERE ref_synchronized<=0 OR ref_synchronized is NULL");
		}else{
			rs=st.executeQuery("select * from contract_info");
		}
		Vector<String> v=new Vector();
		while(rs.next()){
			v.add(rs.getString("name"));
		}
		rs.close();
		st.close();
		return v;		
	}
	
 
	/**
	 * {@inheritDoc}
	 */	
	public Map<String,String> getSecurityNames2RefSync(boolean pendingonly) throws Exception {
		Statement st=this.connection().createStatement();
		ResultSet rs=null;
		if(pendingonly){
			rs=st.executeQuery("select * from security_info WHERE ref_synchronized<=0 OR ref_synchronized is NULL");
		}else{
			 rs=st.executeQuery("select * from security_info");
		}
		LinkedHashMap<String,String>  rtn=new LinkedHashMap();
		while(rs.next()){
			String name=rs.getString("name");
			String marketsector=rs.getString("marketsector");
			rtn.put( name+" "+marketsector,name);
		}
		rs.close();
		st.close();
		return rtn;			
	}
	

	/**
	 * {@inheritDoc}
	 */	
	public Map<String,String> getSecurityName2RefSync(String ticker) throws Exception {
		Statement st=this.connection().createStatement();
		ResultSet rs=null;
		rs=st.executeQuery("select * from security_info where name='"+ticker+"'");
		 
		LinkedHashMap<String,String>  rtn=new LinkedHashMap();
		while(rs.next()){
			String name=rs.getString("name");
			String marketsector=rs.getString("marketsector");
			rtn.put( name+" "+marketsector,name);
		}
		rs.close();
		st.close();
		return rtn;			
	}
	
	
	/**
	 * {@inheritDoc}
	 */	
	public void updateContractReference(String contractname, Map<String,String> fielddata ) throws Exception {
		PreparedStatement ps=this.connection().prepareStatement("SELECT * FROM contract_info WHERE name=?");
		//PreparedStatement ps1=this.connection().prepareStatement("INSERT INTO contract_info(ref_synchronized,first_notice_date,last_trade_date,name) VALUES(?,?,?,?)");
		 	
		SimpleDateFormat sf=new SimpleDateFormat("yyyy-MM-dd");
		
		Date fndate=null;
		Date ltdate=null;
		if(fielddata.get("FUT_LAST_TRADE_DT")!=null){
			ltdate=new Date(sf.parse(fielddata.get("FUT_LAST_TRADE_DT")).getTime());	
		}
		if(fielddata.get("FUT_NOTICE_FIRST")!=null){
			fndate=new Date(sf.parse(fielddata.get("FUT_NOTICE_FIRST")).getTime());	
		}		
		ps.setString(1, contractname);
		ResultSet rs=ps.executeQuery();
		PreparedStatement ps1=null;
		int thisid=0;
		boolean updatemode=true;
		if(rs.next()){
			ps1=this.connection().prepareStatement("UPDATE contract_info SET ref_synchronized=?,first_notice_date=?,last_trade_date=? WHERE name=?");
			PreparedStatement ps11=this.connection().prepareStatement("DELETE FROM contract_info_fields WHERE contract_id=(select id from contract_info where name=? limit 1)" );
			ps11.setString(1, contractname);
			ps11.execute();
			ps11.close();
		
			PreparedStatement st12=this.connection().prepareStatement("select id from contract_info where name=?");
			st12.setString(1,contractname);
			ResultSet rs12=st12.executeQuery();
			if(rs12.next()){
				thisid=rs12.getInt("id");
			}
			rs12.close();
			st12.close();
			
		}else{
			ps1=this.connection().prepareStatement("INSERT INTO contract_info(ref_synchronized,first_notice_date,last_trade_date,name) VALUES(?,?,?,?)",Statement.RETURN_GENERATED_KEYS);
			updatemode=false;
		}
		rs.close();
		
		ps1.setInt(1, 1);
		ps1.setDate(2, fndate);
		ps1.setDate(3, ltdate);
		ps1.setString(4, contractname);
		ps1.executeUpdate();
		if(!updatemode){
			ResultSet rs1 = ps1.getGeneratedKeys(); 
			if(rs1.next()){
				thisid=rs1.getInt(1);				
			}
			rs1.close();	 
		}		
		ps1.close();
		
		
		if(thisid>0){
			PreparedStatement ps3=this.connection().prepareStatement("INSERT INTO contract_info_fields(bb_fieldname,value,contract_id) VALUES(?,?,?)");
			for(Iterator<String> it=fielddata.keySet().iterator();it.hasNext();){
				String bbfield=it.next();
				ps3.setString(1,bbfield);
				ps3.setString(2,fielddata.get(bbfield));
				ps3.setInt(3, thisid);
				ps3.executeUpdate();				
			}
			ps3.close();
		}
	}
		
	
		/**
		 * {@inheritDoc}
		 */	
		public void updateSecurityReference(String securityname, String marketsector, Map<String,String> fielddata ) throws Exception {
			PreparedStatement ps=this.connection().prepareStatement("SELECT * FROM security_info WHERE name=?");			 	
			SimpleDateFormat sf=new SimpleDateFormat("yyyy-MM-dd");			
			Date fndate=null;
			Date ltdate=null;
			
			ps.setString(1, securityname);
			ResultSet rs=ps.executeQuery();
			PreparedStatement ps1=null;
			int thisid=0;
			boolean updatemode=true;
			if(rs.next()){
				ps1=this.connection().prepareStatement("UPDATE security_info SET ref_synchronized=? WHERE name=?");
				PreparedStatement ps11=this.connection().prepareStatement("DELETE FROM security_info_fields WHERE security_id=(select id from security_info where name=? limit 1)" );
				ps11.setString(1, securityname);
				ps11.execute();
				ps11.close();
			
				PreparedStatement st12=this.connection().prepareStatement("select id from security_info where name=?");
				st12.setString(1,securityname);
				ResultSet rs12=st12.executeQuery();
				if(rs12.next()){
					thisid=rs12.getInt("id");
				}
				rs12.close();
				st12.close();
				
			}else{
				ps1=this.connection().prepareStatement("INSERT INTO security_info(ref_synchronized, name,marketsector) VALUES(?,?,?)",Statement.RETURN_GENERATED_KEYS);
				updatemode=false;
			}
			rs.close();
			
			ps1.setInt(1, 1);
			ps1.setString(2, securityname);
			if(!updatemode){
				
				ps1.setString(3, marketsector);
			}
			
			ps1.executeUpdate();
			if(!updatemode){
				ResultSet rs1 = ps1.getGeneratedKeys(); 
				if(rs1.next()){
					thisid=rs1.getInt(1);				
				}
				rs1.close();	 
			}		
			ps1.close();
			
			
			if(thisid>0){
				PreparedStatement ps3=this.connection().prepareStatement("INSERT INTO security_info_fields(bb_fieldname,value,security_id) VALUES(?,?,?)");
				for(Iterator<String> it=fielddata.keySet().iterator();it.hasNext();){
					String bbfield=it.next();
					ps3.setString(1,bbfield);
					ps3.setString(2,fielddata.get(bbfield));
					ps3.setInt(3, thisid);
					ps3.executeUpdate();				
				}
				ps3.close();
			}
			
 	}
	

	/**
	 * {@inheritDoc}
	 */	
	public void addSyncLogs(int bbsyncid, Timestamp start, Timestamp end, String message, String status, String manual_scheduler) throws Exception {
		String dquery="DELETE FROM bbsync_logs WHERE bbsync_id=? AND id NOT IN (select id from (select id from bbsync_logs WHERE bbsync_id=? ORDER BY start_time DESC limit 11) temp_table)";
		PreparedStatement st=this.connection().prepareStatement(dquery);
		st.setInt(1, bbsyncid);
		st.setInt(2, bbsyncid);
		st.executeUpdate();
		st.close();		
		PreparedStatement ps=this.connection().prepareStatement("INSERT INTO bbsync_logs(bbsync_id,start_time,end_time,message,status,manual_scheduler) VALUES(?,?,?,?,?,?)");
		ps.setInt(1, bbsyncid);
		ps.setTimestamp(2, start);
		ps.setTimestamp(3, end);
		ps.setString(4,message);
		ps.setString(5,status);
		ps.setString(6,manual_scheduler);
		
		ps.executeUpdate();
		ps.close();
		
	}
	
	
	/**
	 * {@inheritDoc}
	 */		
	public void removeScheduleTicker(int bbsync_id, String ticker) throws Exception {
		PreparedStatement ps=this.connection().prepareStatement("DELETE FROM bbsync_contracts WHERE bbsync_id=? AND contract=?" );
		ps.setInt(1, bbsync_id);
		ps.setString(2, ticker.trim());
		ps.executeUpdate();
		ps.close();
	}
	
	
	/**
	 * {@inheritDoc}
	 */	
	public int saveSchedule(	
			int id,
			String name,
			String mkt_secdb,			
			String dateoption,
			Date datefrom,
			Date dateto,
			int datenumber,
			Vector fields,		
			String contracts,
			BBSyncTrigger t,
			String marketsector,
			String timezone
	) throws Exception {

		log.debug("saveSchedule() called:");
		String query1;
		PreparedStatement ps;
		if(id>0){			
			query1="UPDATE bbsync SET name=?,is_mkt_securitydb=?,date_option=?,date_recentnumber=?,date_from=?,date_to=?,tickers=?,trigger_type=?,fields=?,trigger_time=?,trigger_days=?,trigger_day=?,marketsector=?,trigger_week=?,trigger_dailyhour=?,timezone=? WHERE id=?";
			ps=this.connection().prepareStatement(query1);
		}else{
			query1="INSERT INTO bbsync(name,is_mkt_securitydb,date_option,date_recentnumber,date_from,date_to,tickers,trigger_type,fields,trigger_time,trigger_days,trigger_day,marketsector,trigger_week,trigger_dailyhour,timezone) VALUES(?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?) ";
			ps=this.connection().prepareStatement(query1,Statement.RETURN_GENERATED_KEYS);
		}
		
		log.debug("connection:"+connection());
		
		log.debug("ps:"+ps);		
		log.debug("t:"+t);
		log.debug("query:"+query1);
		log.debug("id:"+id);
		ps.setString(1, name);			
		ps.setString(2, mkt_secdb);
		ps.setString(3, dateoption);
		ps.setInt(4, datenumber);
		 
		ps.setDate(5, datefrom);
		ps.setDate(6, dateto);
		 
		//ps.setString(7, contracts);
		ps.setString(7, "");
		
		ps.setInt(8, t.getSynctype());		
		ps.setString(9, ""); // to be removed later.
		ps.setString(10,  t.getTime());
		ps.setString(11, t.getDays());
		//ps.setString(13, t.getMonths());
	 
		ps.setInt(12, t.getDay());
		ps.setString(13, marketsector);
		ps.setInt(14, (t.getWeek()!=null)?t.getWeek():0	);
		ps.setInt(15, (t.getDailyhour()!=null)?t.getDailyhour():0);
		ps.setString(16, timezone);
		if(id>0){
			ps.setInt(17, id);
			ps.executeUpdate();
		}else{
			ps.executeUpdate();		 
			ResultSet rs = ps.getGeneratedKeys(); 
			if(rs.next()){
				id=rs.getInt(1);				
			}
			rs.close();	 
		}	
		ps.close();
		
		if(id>0){
			Statement st=this.connection().createStatement();
			st.execute("DELETE FROM bbsync_fields WHERE bbsync_id="+id);
			st.close();
			PreparedStatement ps2=this.connection().prepareStatement("INSERT into bbsync_fields(bbsync_id,field_id) values(?,?)");
			for(int a=0;a<fields.size();a++){
				ps2.setInt(1, id);
				ps2.setInt(2, Integer.parseInt((String)fields.get(a)));
				ps2.executeUpdate();
			}
			ps2.close();
			
			
			ArrayList tickers= parseFreeTextTokens(contracts);
			Statement st1=this.connection().createStatement();
			st1.execute("DELETE FROM bbsync_contracts WHERE bbsync_id="+id);
			st1.close();
			PreparedStatement ps3=this.connection().prepareStatement("INSERT into bbsync_contracts(bbsync_id,contract) values(?,?)");
			for(int a=0;a<tickers.size();a++){
				ps3.setInt(1, id);
				ps3.setString(2, ((String)tickers.get(a)).trim());
				ps3.executeUpdate();
			}
			ps3.close();
			
		}
		return id;
	}
	
	
	/**
	 * {@inheritDoc}
	 */	
	public void deleteQuery(int id) throws Exception {
		
		if(id>0){
			Statement st=this.connection().createStatement();
			st.execute("DELETE FROM bbsync_fields WHERE bbsync_id="+id);
			st.close();
			
			Statement st1=this.connection().createStatement();
			st1.execute("DELETE FROM bbsync_contracts WHERE bbsync_id="+id);
			st1.close();
		
			Statement st2=this.connection().createStatement();
			st2.execute("DELETE FROM bbsync WHERE id="+id);
			st2.close();
		}
	}


	/**
	 * {@inheritDoc}
	 */	
	public void peerStarted(String peername, long sessionid, java.util.Date time) throws Exception  {
		    Timestamp ts=new Timestamp(time.getTime());
			PreparedStatement ps=this.connection().prepareStatement("INSERT INTO peer_sessions(peername, sessionid,started_time) VALUES (?,?,?)");
			ps.setString(1, peername);
			ps.setLong(2, sessionid);
			ps.setTimestamp(3,ts );
			ps.executeUpdate();
			ps.close();
	}


	/**
	 * {@inheritDoc}
	 */	
	public void peerStopped(String peername,long sessionid, java.util.Date time) throws Exception  {
		Timestamp ts=new Timestamp(time.getTime());
		PreparedStatement ps=this.connection().prepareStatement("UPDATE peer_sessions SET stopped_time=? WHERE peername=? AND sessionid=?");
		ps.setTimestamp(1,ts );
		ps.setString(2, peername);
		ps.setLong(3, sessionid);	
		ps.executeUpdate();
		ps.close();
	}
	
	/**
	 * Parse free text to list by token
	 * @param stringToken string token
	 * @return text separated by token in list
	 */		
	private ArrayList parseFreeTextTokens(String stringToken){
		ArrayList rtn=new ArrayList();
		if(stringToken!=null && !stringToken.equals("")){
			String pline=fieldSeparator4FreeText(stringToken);
			StringTokenizer st=new StringTokenizer(stringToken,pline);
			while(st.hasMoreTokens()){
				rtn.add(st.nextToken());
			}
		}
		return rtn;
	}	
	
	/**
	 * Get field separator from free text
	 * @param stringToken string token
	 * @return field separator
	 */		
	private String fieldSeparator4FreeText(String stringToken){
		return (stringToken.indexOf(",")>=0)?",":"\r\n";
	}	
	
}
