/******************************************************************************
*
* Copyright: Intellectual Property of Four Elements Capital Pte Ltd, Singapore.
* All rights reserved.
*
******************************************************************************/

package com.fourelementscapital.db.mariadb;

import java.util.Map;

import com.fourelementscapital.db.ConstructQueryDB;

/**
 * Construct Query MariaDB DAO implementation
 */	
public class ConstructQueryDBMariaDB extends ConstructQueryDB {

	
	/**
	 * {@inheritDoc}
	 */	
	 public String constructDateInputQuery(Map rtnobj) {
		 
	   if (rtnobj == null) {
		   return null;
	   }
	   
	   String q = null;

	   String filtertype = rtnobj.get("filtertype") == null ? "" : rtnobj.get("filtertype").toString();
	   if ("daterange".equals(filtertype)) {
		   String dobj1Success = rtnobj.get("dobj1Success").toString();
		   String dobj2Success = rtnobj.get("dobj2Success").toString();
		   String cdatefield = rtnobj.get("cdatefield").toString();
		   String dobj1Sqldate = rtnobj.get("dobj1Sqldate").toString();
		   String dobj2Sqldate = rtnobj.get("dobj2Sqldate").toString();
		   if ("true".equals(dobj1Success) && "true".equals(dobj2Success)) {
			   q = cdatefield+" BETWEEN '"+ dobj1Sqldate +"' AND '"+ dobj2Sqldate +"'";
		   }
	   }
	   else if ("datefrom".equals(filtertype)) {
		   String dobj1Success = rtnobj.get("dobj1Success").toString();
		   String cdatefield = rtnobj.get("cdatefield").toString();
		   String dobj1Sqldate = rtnobj.get("dobj1Sqldate").toString();
		   if ("true".equals(dobj1Success)) {
			   q = cdatefield+" BETWEEN '"+ dobj1Sqldate +"' AND now() ";	
		   }
	   }
	   else if ("number".equals(filtertype)) {	   
		   String filtervalue = rtnobj.get("filtervalue").toString();
		   String numbr = rtnobj.get("number").toString();
		   String cdatefield = rtnobj.get("cdatefield").toString();
		   if ("ndays".equals(filtervalue)) {
			   q = cdatefield+">=DATE_ADD(NOW(), interval -"+numbr+" day)";
		   }
		   else if ("nweeks".equals(filtervalue)) {
			   q = cdatefield+">=DATE_ADD(NOW(), interval -"+numbr+" week)";
		   }
		   else if ("nmonths".equals(filtervalue)) {
			   q = cdatefield+">=DATE_ADD(NOW(), interval -"+numbr+" month)";
		   }
		   else if ("nyears".equals(filtervalue)) {
			   q = cdatefield+">=DATE_ADD(NOW(), interval -"+numbr+" year)";
		   }			   
	   }
	   else {
		   Object filtervalue = rtnobj.get("filtervalue");
		   String cdatefield = rtnobj.get("cdatefield").toString();
		   if (filtervalue!=null && !"".equals(filtervalue.toString()) && cdatefield!=null && !"".equals(cdatefield.toString())) {
			   q = constructRecentQuery(cdatefield, filtervalue.toString());
		   }
		   else {
			   q = null;
		   }
	   }
	   
	   return q;
	}   
	   

	/**
	 * {@inheritDoc}
	 */	 
	 public String constructRecentQuery(String field, String dateQuery) {
		 
		 if (field == null || "".equals(field)) {
			return null;
		 }
		 if (dateQuery == null || "".equals(dateQuery)) {
			return null;
		 }
		 
		 String q = null;
		 if ("last1Hour".equals(dateQuery)) {
			 q = field + ">=DATE_ADD(NOW(), interval -1 hour)";
		 }
		 else if ("last2Hour".equals(dateQuery)) {
			 q = field + ">=DATE_ADD(NOW(), interval -2 hour)";
		 }
		 else if ("last5Hour".equals(dateQuery)) {
			 q = field + ">=DATE_ADD(NOW(), interval -5 hour)";
		 }		 
		 else if ("last10Hour".equals(dateQuery)) {
			 q = field + ">=DATE_ADD(NOW(), interval -10 hour)";
		 }		 
		 else if ("last1Day".equals(dateQuery)) {
			 q = field + ">=DATE_ADD(NOW(), interval -1 day)";
		 }		 
		 else if ("last2Day".equals(dateQuery)) {
			 q = field + ">=DATE_ADD(NOW(), interval -2 day)";
		 }		 
		 else if ("last3Day".equals(dateQuery)) {
			 q = field + ">=DATE_ADD(NOW(), interval -3 day)";
		 }		 
		 else if ("last1Week".equals(dateQuery)) {
			 q = field + ">=DATE_ADD(NOW(), interval -1 week)";
		 }		 
		 else if ("last10Week".equals(dateQuery)) {
			 q = field + ">=DATE_ADD(NOW(), interval -10 week)";
		 }		 
		 else if ("last1Month".equals(dateQuery)) {
			 q = field + ">=DATE_ADD(NOW(), interval -1 month)";
		 }		 
		 else if ("last2Month".equals(dateQuery)) {
			 q = field + ">=DATE_ADD(NOW(), interval -2 month)";
		 }		 
		 else if ("last3Month".equals(dateQuery)) {
			 q = field + ">=DATE_ADD(NOW(), interval -3 month)";
		 }		 
		 else if ("last6Month".equals(dateQuery)) {
			 q = field + ">=DATE_ADD(NOW(), interval -6 month)";
		 }		 
		 else if ("last12Month".equals(dateQuery)) {
			 q = field + ">=DATE_ADD(NOW(), interval -12 month)";
		 }		 
		 else if ("last18Month".equals(dateQuery)) {
			 q = field + ">=DATE_ADD(NOW(), interval -18 month)";
		 }		 
		 else if ("last24Month".equals(dateQuery)) {
			 q = field + ">=DATE_ADD(NOW(), interval -24 month)";
		 }		 
		 else if ("last3Year".equals(dateQuery)) {
			 q = field + ">=DATE_ADD(NOW(), interval -3 year)";
		 }		 
		 else if ("last4Year".equals(dateQuery)) {
			 q = field + ">=DATE_ADD(NOW(), interval -4 year)";
		 }		 
		 else if ("last5Year".equals(dateQuery)) {
			 q = field + ">=DATE_ADD(NOW(), interval -5 year)";
		 }		 
		 else if ("last10Year".equals(dateQuery)) {
			 q = "trigger_datetime>=DATE_ADD(NOW(), interval -10 year)";
		 }		 
		 return q;
	 }
	 
	 
		/**
		 * {@inheritDoc}
		 */		 
	   public String constructQueueHistoryQuery(Map rtnobj) {
		   
		   if (rtnobj == null) {
			   return null;
		   }
		   
		   String q = null;
	
		   //q = rtnobj.get("query") == null ? "null" : rtnobj.get("query").toString();
		   q = constructDateInputQuery(rtnobj);
		   String statfilter = rtnobj.get("statfilter") == null ? "" : rtnobj.get("statfilter").toString();
		   String scd_typefilter = rtnobj.get("scd_typefilter") == null ? "" : rtnobj.get("scd_typefilter").toString();
		   
		   if ("true".equals(rtnobj.get("queryCheck").toString())) {
			   
			   if(!"".equals(statfilter)){
		           if("{null}".equals(statfilter)){
		               q+=" AND a.status IS NULL";
		           }else if("#success".equals(statfilter)){
		               q+=" AND (a.status<>'success' OR a.status IS NULL )";
		           }else {
		               q+=" AND a.status='"+statfilter+"'";
		           }	
			   }
	
			   q += (!"".equals(scd_typefilter)) ? " AND b.taskuid='"+ scd_typefilter +"'" : ""; 
			   
			   String fld1 = rtnobj.get("fld1") == null ? "" : rtnobj.get("fld1").toString();
			   String scd_filterfieldval = rtnobj.get("scd_filterfieldval") == null ? "" : rtnobj.get("scd_filterfieldval").toString();
			   
			   String q1 = "";
		        if(!"".equals(fld1) && !"tag".equals(fld1) && !"".equals(scd_filterfieldval)){             
		            String val1=scd_filterfieldval;
		            if("name".equals(fld1)){q1=" b.name LIKE '%"+val1+"%' ";}
		            if("scheduler_id".equals(fld1)){q1=" a.scheduler_id="+val1+"";}  
		            q += ("".equals(q1)) ? "" : " AND "+ q1 +" ";    
		        }
	
		        String scd_filterddval = rtnobj.get("scd_filterddval") == null ? "" : rtnobj.get("scd_filterddval").toString();
		        if("host".equals(fld1) && !"".equals(scd_filterddval)) {
		            q1 = " a.host='"+ scd_filterddval +"'";
		            q += ("".equals(q1)) ? "" : " AND "+ q1 +" ";  
		        }
		        
		        if("tag".equals(fld1) && !"".equals(scd_filterddval)) {
		            String val1 = rtnobj.get("scd_filterddval") == null ? "" : rtnobj.get("scd_filterddval").toString();
		            q1=" a.scheduler_id IN(SELECT scheduler_id from scheduler_tags where tag_id="+val1+") ";
		            q += ("".equals(q1)) ? "" : " AND "+ q1 +" ";     
		        }			   
		   }	   
	
		   return q;
	   }      
	   
}
