/******************************************************************************
*
* Copyright: Intellectual Property of Four Elements Capital Pte Ltd, Singapore.
* All rights reserved.
*
******************************************************************************/

package com.fourelementscapital.db.mariadb;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Vector;

import org.apache.commons.dbutils.BasicRowProcessor;

import com.fourelementscapital.db.IExecDB;


/**
 * IExec MariaDB DAO implementation
 */	
public class IExecDBMariaDB extends IExecDB{

	 
	/**
	 * {@inheritDoc}
	 */	
	public List listGroups() throws Exception {
			
			//PreparedStatement ps=this.connection().prepareStatement("Select * from `ie_group` ORDER BY disp_order");
			PreparedStatement ps=this.connection().prepareStatement("Select * from ie_group ORDER BY disp_order");
			
			ResultSet rs=ps.executeQuery();
			Vector rtn=new Vector();
			while(rs.next()){			
				rtn.add(new BasicRowProcessor().toMap(rs));
			}
			return rtn;
		 
	}
	
	
	/**
	 * {@inheritDoc}
	 */	
	public void setGroupOrder( Vector groupids) throws Exception {		
		 
		//PreparedStatement ps1=this.connection().prepareStatement("UPDATE `ie_group` SET disp_order=? WHERE group_uid=?");
		PreparedStatement ps1=this.connection().prepareStatement("UPDATE ie_group SET disp_order=? WHERE group_uid=?");
		int count=0;
		for(Iterator<String> i=groupids.iterator();i.hasNext();){
			ps1.setInt(1, count++);
			ps1.setString(2, i.next());			
			ps1.executeUpdate();
		}
	}
	
	
	/**
	 * {@inheritDoc}
	 */		
	public List listFolders( ) throws Exception {
		
		PreparedStatement ps=null;		 
		ps=this.connection().prepareStatement("Select * FROM ie_folder ORDER BY folder_name");		 
		
		ResultSet rs=ps.executeQuery();
		Vector rtn=new Vector();
		while(rs.next()){			
			rtn.add(new BasicRowProcessor().toMap(rs));
		}
		return rtn;
	}
	
	
	/**
	 * {@inheritDoc}
	 */	
	public String getFolderName(int folder_id) throws Exception {
		PreparedStatement ps=this.connection().prepareStatement("Select folder_name  FROM  ie_folder  WHERE id=?");		
		ps.setInt(1,folder_id);
		ResultSet rs=ps.executeQuery();
		String rtn=null;
		while(rs.next()){			
			rtn=rs.getString(1);
		}
		return rtn;		
	}
	
	
	/**
	 * {@inheritDoc}
	 */	
	public void renameStrategy(int strategy_id, String strategy_name, String file_name) throws Exception {
		PreparedStatement ps=this.connection().prepareStatement("UPDATE ie_strategy SET strategy_name=?,file_name=? WHERE id=?");
		
		ps.setString(1, strategy_name);
		ps.setString(2, file_name);
		ps.setInt(3,strategy_id);
		
		ps.executeUpdate();
	}
	
	
	/**
	 * {@inheritDoc}
	 */	
	public void renameParentStrategy(int strategy_id, String strategy_name, String file_name, String old_strategy_name) throws Exception {
		PreparedStatement ps=this.connection().prepareStatement("UPDATE ie_strategy SET strategy_name=?,file_name=? WHERE id=?");		
		ps.setString(1, strategy_name);
		ps.setString(2, file_name);
		ps.setInt(3,strategy_id);		
		ps.executeUpdate();
		
		PreparedStatement ps2=this.connection().prepareStatement("UPDATE ie_strategy SET parent_strategy=? WHERE parent_strategy=?");		
		ps2.setString(1, strategy_name);
		ps2.setString(2, old_strategy_name);
		ps2.executeUpdate();
		
	}
	
	
	/**
	 * {@inheritDoc}
	 */	
	public void moveFolder(int folder_id, String new_group_id) throws Exception{
		PreparedStatement ps=this.connection().prepareStatement("UPDATE ie_folder SET group_uid = ? WHERE  id=?");
		ps.setString(1, new_group_id);
		ps.setInt(2,folder_id);
		ps.executeUpdate();
		ps.close();
	}
	
	
	/**
	 * {@inheritDoc}
	 */	
	public void updateStrategyFolder(int strategy_id,int new_folder_id) throws Exception {
		PreparedStatement ps=this.connection().prepareStatement("UPDATE ie_strategy SET folder_id=? WHERE id=?");
		ps.setInt(1,new_folder_id);
		ps.setInt(2, strategy_id);
		ps.executeUpdate();
		ps.close(); 
	}
	
	
	/**
	 * {@inheritDoc}
	 */	
	public List listStrategies() throws Exception {
		
		PreparedStatement ps=this.connection().prepareStatement("Select a.*,b.folder_name  FROM  ie_strategy as a left outer join ie_folder as b on a.folder_id=b.id order by a.parent_strategy, a.strategy_name");
		
		ResultSet rs=ps.executeQuery();
		Vector rtn=new Vector();
		while(rs.next()){			
			rtn.add(new BasicRowProcessor().toMap(rs));
		}
		return rtn;
	}

	
	/**
	 * {@inheritDoc}
	 */	
	public Map getStrategy(int strategy_id) throws Exception {
		
		//PreparedStatement ps=this.connection().prepareStatement("Select *  FROM  r_function WHERE  id=?");
		PreparedStatement ps=this.connection().prepareStatement("Select a.*,b.folder_name FROM  ie_strategy as a left outer join ie_folder as b on a.folder_id=b.id WHERE  a.id=? ORDER BY a.strategy_name ");
		
		 
		ps.setInt(1, strategy_id);
		ResultSet rs=ps.executeQuery();
		
		Map rtn=null;
		if(rs.next()){			
			rtn=new BasicRowProcessor().toMap(rs);
		}
		return rtn;
	}
	
	
	/**
	 * {@inheritDoc}
	 */	
	public Vector getUniqueContracts(String strategy_name) throws Exception {
		PreparedStatement ps=this.connection().prepareStatement("Select DISTINCT contract FROM ie_parameter WHERE strategy_name=? ");
		ps.setString(1,strategy_name);
		ResultSet rs=ps.executeQuery();
		
		Vector rtn=new Vector();
		while(rs.next()){			
			rtn.add(rs.getString("contract"));
		}
		return rtn;
	}
	
	
	/**
	 * {@inheritDoc}
	 */	
	public void addParameters(ArrayList<Map> data,String strategy_name) throws Exception {
		PreparedStatement ps1=this.connection().prepareStatement("DELETE FROM ie_parameter WHERE strategy_name=?");
		ps1.setString(1,strategy_name);
		ps1.executeUpdate();
		
		
		PreparedStatement ps=this.connection().prepareStatement("INSERT INTO ie_parameter(strategy_name,contract,placeholder,value) VALUES(?,?,?,?)");
		for(Iterator  i=data.iterator();i.hasNext();){
			Map row=(Map)i.next();
			ps.setString(1, (String)row.get("strategy_name"));
			ps.setString(2, (String)row.get("contract"));
			ps.setString(3, (String)row.get("placeholder"));
			ps.setString(4, (String)row.get("value"));
			ps.executeUpdate();
		}
		
	}
	
	
	/**
	 * {@inheritDoc}
	 */	
	public void removeContract(String strategy_name, String contract) throws Exception {
		PreparedStatement ps1=this.connection().prepareStatement("DELETE FROM ie_parameter WHERE strategy_name=? AND contract=?");
		ps1.setString(1,strategy_name);
		ps1.setString(2,contract);
		ps1.executeUpdate();
	}
	
	
	/**
	 * {@inheritDoc}
	 */	
	public Map getParameterValues(String strategy_name, String contract) throws Exception {
		PreparedStatement ps=this.connection().prepareStatement("Select placeholder,value FROM ie_parameter WHERE strategy_name=? AND contract=?");
		ps.setString(1, strategy_name);
		ps.setString(2, contract);
		ResultSet rs=ps.executeQuery();
		HashMap h=new HashMap();
		while(rs.next()){
			h.put(rs.getString("placeholder"), rs.getString("value"));
		}
		rs.close();
		ps.close();
		
		return h;
	}
	
	
	/**
	 * {@inheritDoc}
	 */	
	public Map getStrategy(String strategy_name) throws Exception {
		
		PreparedStatement ps=this.connection().prepareStatement("Select *  FROM  ie_strategy WHERE  strategy_name=?");
		ps.setString(1, strategy_name);
		ResultSet rs=ps.executeQuery();		
		Map rtn=null;
		if(rs.next()){			
			rtn=new BasicRowProcessor().toMap(rs);
		}
		return rtn;
	}
	
	
	/**
	 * {@inheritDoc}
	 */	
	public Vector getStrategies(Vector strategy_names) throws Exception {
		
		//PreparedStatement ps=this.connection().prepareStatement("Select *  FROM  r_function WHERE  id=?");
		
		String sfunc="''";
		for(Iterator i=strategy_names.iterator();i.hasNext();){
			sfunc+=",'"+i.next()+"'";
		}
		PreparedStatement ps=this.connection().prepareStatement("Select *  FROM  ie_strategy WHERE strategy_name IN ("+sfunc+") ORDER BY strategy_name ");
 

		ResultSet rs=ps.executeQuery();
		
		Vector rtn=new Vector();
		while(rs.next()){			
			rtn.add(new BasicRowProcessor().toMap(rs));
		}
		return rtn;
	}

	
	/**
	 * {@inheritDoc}
	 */	
	public int createStrategy(int folder_id,String strategy_name, String path) throws Exception {
		PreparedStatement ps=this.connection().prepareStatement("INSERT INTO ie_strategy(folder_id,strategy_name,file_name) VALUES(?,?,?)",Statement.RETURN_GENERATED_KEYS);
		ps.setInt(1,folder_id);
		ps.setString(2, strategy_name);
		ps.setString(3, path);
		ps.executeUpdate();
		int id=0;
		ResultSet generatedKeys = ps.getGeneratedKeys();
		
        if (generatedKeys.next()) {
            id=generatedKeys.getInt(1);
        }		
		ps.close();	

		return id; 
		
	}
	
	
	/**
	 * {@inheritDoc}
	 */	
	public int createChildStrategy(String parent_name,String strategy_name) throws Exception {
		PreparedStatement ps=this.connection().prepareStatement("INSERT INTO ie_strategy(parent_strategy,strategy_name) VALUES(?,?)",Statement.RETURN_GENERATED_KEYS);
		ps.setString(1,parent_name);
		ps.setString(2, strategy_name);
	 
		ps.executeUpdate();
		int id=0;
		ResultSet generatedKeys = ps.getGeneratedKeys();
		
        if (generatedKeys.next()) {
            id=generatedKeys.getInt(1);
        }		
		ps.close();	

		return id; 
	}
	

	/**
	 * {@inheritDoc}
	 */	
	public int getFolderID(String folder_name) throws Exception {
		
		PreparedStatement ps=null;		 
		ps=this.connection().prepareStatement("Select * FROM ie_folder WHERE folder_name=?");
		ps.setString(1, folder_name);
		ResultSet rs=ps.executeQuery();
		int rtn=0;
		if(rs.next()){			
		 rtn=rs.getInt("id");
		}
		return rtn;
	}
	
	
	/**
	 * {@inheritDoc}
	 */	
	public int createFolder(String folder, String new_group_id) throws Exception{		
		
		PreparedStatement ps=this.connection().prepareStatement("INSERT INTO ie_folder(folder_name,group_uid) VALUES(?,?)",Statement.RETURN_GENERATED_KEYS);
		ps.setString(1, folder);
		ps.setString(2, new_group_id);				 
		ps.executeUpdate();
		ResultSet generatedKeys = ps.getGeneratedKeys();
		int id=0;
	    if (generatedKeys.next()) {
	    	id=generatedKeys.getInt(1);
	    }	    
		ps.close();
		return id; 
	}


	/**
	 * {@inheritDoc}
	 */	
	public List listOfFolders(String group_id) throws Exception {
		
		PreparedStatement ps=null;		 
		ps=this.connection().prepareStatement("Select * FROM ie_folder WHERE group_uid=?  ORDER BY folder_name");		 
		ps.setString(1, group_id);
		ResultSet rs=ps.executeQuery();
		Vector rtn=new Vector();
		while(rs.next()){			
			rtn.add(new BasicRowProcessor().toMap(rs));
		}
		return rtn;
	}
	
	
	/*
	public Vector getContractTree() throws Exception {
		StringBuffer sb = new StringBuffer();
		sb.append("SELECT contract,commodity as commodity FROM trading.tblContractAllocation  WHERE commodity in (SELECT commodity from tradingRef.commodityRef) ");
		sb.append("AND date= ");
		sb.append("CASE DAYOFWEEK(SUBDATE(CURRENT_DATE, 1)) ");
		sb.append("  WHEN 1 THEN SUBDATE(CURRENT_DATE, 3) ");
		sb.append("  WHEN 7 THEN SUBDATE(CURRENT_DATE, 2) ");
		sb.append("  ELSE SUBDATE(CURRENT_DATE, 1) ");
		sb.append("END ");
		sb.append("ORDER BY commodity, year ASC, month ASC ");
		PreparedStatement ps=this.connection().prepareStatement(sb.toString());	 
		ResultSet rs=ps.executeQuery();
		Vector rtn=new Vector();
		while(rs.next()){			
			rtn.add(new BasicRowProcessor().toMap(rs));
		}
		return rtn;
	}
	*/
	
	
	/**
	 * {@inheritDoc}
	 */	
	public Vector getContractTree() throws Exception {
		StringBuffer sb = new StringBuffer();
		/*sb.append("SELECT contract,commodity as commodity FROM " + DB_NAME_TRADING + ".dbo.tblContractAllocation  WHERE commodity in (SELECT commodity from " + DB_NAME_TRADINGREF + ".dbo.commodityRef) ");
		sb.append("AND date= ");
		sb.append("CASE DAYOFWEEK(SUBDATE(CURRENT_DATE, 1)) ");
		sb.append("  WHEN 1 THEN SUBDATE(CURRENT_DATE, 3) ");
		sb.append("  WHEN 7 THEN SUBDATE(CURRENT_DATE, 2) ");
		sb.append("  ELSE SUBDATE(CURRENT_DATE, 1) ");
		sb.append("END ");
		sb.append("ORDER BY commodity, year ASC, month ASC ");*/

		sb.append("SELECT contract,commodity as commodity ");
		sb.append("FROM trading.dbo.tblContractAllocation ");
		sb.append("WHERE commodity in (SELECT commodity from tradingRef.dbo.commodityRef) "); 
		sb.append("AND date= CASE DATEPART(w,DATEADD(DAY,1,CONVERT(date, GETDATE()))) ");
		sb.append("WHEN 1 THEN DATEADD(DAY,-3,CONVERT(date, GETDATE())) ");
		sb.append("WHEN 7 THEN DATEADD(DAY,-2,CONVERT(date, GETDATE())) ");
		sb.append("ELSE DATEADD(DAY,-1,CONVERT(date, GETDATE())) END ");
		sb.append("ORDER BY commodity, year ASC, month ASC");

		PreparedStatement ps=this.connection().prepareStatement(sb.toString());	 
		ResultSet rs=ps.executeQuery();
		Vector rtn=new Vector();
		while(rs.next()){			
			rtn.add(new BasicRowProcessor().toMap(rs));
		}
		return rtn;
	}
	
	
	/*	
	public Vector<String> getCommodityTree() throws Exception {
		String query="SELECT commodity from tradingRef.commodityRef ORDER BY orderNo";
		PreparedStatement ps=this.connection().prepareStatement(query);	 
		ResultSet rs=ps.executeQuery();
		Vector rtn=new Vector();
		while(rs.next()){			
			//rtn.add(new BasicRowProcessor().toMap(rs));
			rtn.add(rs.getString("commodity"));
		}
		return rtn;
	}
	*/
	
	
	/**
	 * {@inheritDoc}
	 */	
	public Vector<String> getCommodityTree() throws Exception {
		String query="SELECT commodity from " + DB_NAME_TRADINGREF + ".dbo.commodityRef ORDER BY orderNo";
		PreparedStatement ps=this.connection().prepareStatement(query);	 
		ResultSet rs=ps.executeQuery();
		Vector rtn=new Vector();
		while(rs.next()){			
			//rtn.add(new BasicRowProcessor().toMap(rs));
			rtn.add(rs.getString("commodity"));
		}
		return rtn;
	}	


	/**
	 * {@inheritDoc}
	 */	
	public boolean isParent(String strategy_name) throws Exception {
	    boolean rtn=false;
	    
		String query="select * from ie_strategy where parent_strategy=?";
		PreparedStatement ps=this.connection().prepareStatement(query);
		ps.setString(1, strategy_name);
		ResultSet rs=ps.executeQuery();
		 
		if(rs.next()){			
			//rtn.add(new BasicRowProcessor().toMap(rs));
			 rtn=true;
		}
		return rtn;
		
	}
}
