/******************************************************************************
*
* Copyright: Intellectual Property of Four Elements Capital Pte Ltd, Singapore.
* All rights reserved.
*
******************************************************************************/

package com.fourelementscapital.db.mariadb;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;

import org.apache.commons.dbutils.BasicRowProcessor;

import com.fourelementscapital.db.InfrastructureDB;

/**
 *  Infrastructure MariaDB DAO implementation
 */
public class InfrastructureDBMariaDB extends InfrastructureDB {
	

	/**
	 * {@inheritDoc}
	 */	 
	 public Map getThemes4Users(String user) throws Exception {
		 //String query="select * from tblTeamOrganization where theme in ('bb','port','itools')";
		 
		 String query="select * from tblTeamOrganization ";
		 PreparedStatement ps=this.connection().prepareStatement(query);
		 ResultSet rs=ps.executeQuery();	
		 ResultSetMetaData rm= rs.getMetaData();
		 ArrayList<String> columns=new ArrayList();
		 for(int i=0;i<rm.getColumnCount();i++){
			 String column=rm.getColumnName(i+1);
			 columns.add(column);
		 }		
		 TreeMap<String, String> themes=new TreeMap();
		 while(rs.next()){
			 
				String leter=rs.getString(user);
				String theme=rs.getString("theme");
				if(leter!=null && !leter.trim().equals("")){
					themes.put(theme, leter);					
				}
		 }
		 return themes;
	 }
	 
	/**
	 * {@inheritDoc}
	 */	
	 public Map getTeamOrg(String themes, List hierarchy) throws Exception {
		 //String query="select * from tblTeamOrganization where theme in ('bb','port','itools')";
		 
		 String query="select * from tblTeamOrganization where theme in ("+themes+")";
		 
		 PreparedStatement ps=this.connection().prepareStatement(query);
		 ResultSet rs=ps.executeQuery();	
		 ResultSetMetaData rm= rs.getMetaData();
		 ArrayList<String> columns=new ArrayList();
		 for(int i=0;i<rm.getColumnCount();i++){
			 String column=rm.getColumnName(i+1);
			 columns.add(column);
		 }		
		 TreeMap<String, String> usrs=new TreeMap();
		 while(rs.next()){
			for(String user:columns){
				String val=rs.getString(user);
				if(val!=null && !val.trim().equals("") ){
					if(usrs.get(user)!=null){
						String old=usrs.get(user);
						int oldorder=hierarchy.indexOf(old);
						int neworder=hierarchy.indexOf(val);
						if(neworder<oldorder){
							usrs.put(user,val);
						}
					}else{
						usrs.put(user,val);
					}
				}
			}
		 }
		 return usrs;
	 }	 
	 
	/**
	 * {@inheritDoc}
	 */	
	public void updateBloomberDailyCounter(String date, int count) throws Exception {
	    String query="INSERT INTO tblInfrastructureInfo (date, BBGDailyCounter) VALUES ("+ date +","+ count +") ON DUPLICATE KEY UPDATE BBGDailyCounter=BBGDailyCounter+" + count;			   
		PreparedStatement ps=this.connection().prepareStatement(query);
		ps.execute();
		ps.close();
	}	

	/**
	 * {@inheritDoc}
	 */	
	public List<String> getThemeByUsernameAccess(String username) throws Exception {
		String query="select 'thm-'+theme from tblTeamOrganization where "+username+" IN ('X','B','C'))";
		PreparedStatement ps=this.connection().prepareStatement(query);
		ResultSet rs=ps.executeQuery();
		ArrayList rtn=new ArrayList();
		while(rs.next()){
			rtn.add(new BasicRowProcessor().toMap(rs));
		}
		rs.close();
		ps.close();
		return rtn;	
	}	
	
	/**
	 * {@inheritDoc}
	 */	
	 public List getAccessByAppAndUsername(String application, String username) throws Exception {
		 
		 List al=new ArrayList();
		 
		 String query="SELECT access FROM ApplicationUserAccess WHERE application=? and username=?";
			
		 PreparedStatement ps=this.connection().prepareStatement(query);
		 ps.setString(1, application);
		 ps.setString(2, username);
		 
		 ResultSet rs=ps.executeQuery();		 
		 while(rs.next()){			 
			 al.add(rs.getString(1));
		 }
		 rs.close();
		 ps.close();
		 return al;
	 }

}
