/******************************************************************************
*
* Copyright: Intellectual Property of Four Elements Capital Pte Ltd, Singapore.
* All rights reserved.
*
******************************************************************************/

package com.fourelementscapital.db.mariadb;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.Statement;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Vector;

import org.apache.commons.dbutils.BasicRowProcessor;
import org.apache.logging.log4j.Logger;
import org.apache.logging.log4j.LogManager;

import com.fourelementscapital.db.RFunctionDB;
import com.fourelementscapital.db.SchedulerDB;
import com.fourelementscapital.db.vo.PeerPackage;


/**
 * RFunction MariaDB DAO implementation
 */	
public class RFunctionDBMariaDB extends RFunctionDB {
	
	
	private Logger log=LogManager.getLogger(RFunctionDBMariaDB.class.getName());
	
	
	/**
	 * {@inheritDoc}
	 */		
	public String getFolderName(int folder_id) throws Exception {
		PreparedStatement ps=this.connection().prepareStatement("Select folder_name  FROM  folder  WHERE id=?");		
		ps.setInt(1,folder_id);
		ResultSet rs=ps.executeQuery();
		String rtn=null;
		while(rs.next()){			
			rtn=rs.getString(1);
		}
		return rtn;		
	}
	
	
	/**
	 * {@inheritDoc}
	 */	
	public void updateWikiDone(int function_id, int done) throws Exception {
		String query="UPDATE r_function set is_wiki_done=? where  id=?";
		PreparedStatement ps=this.connection().prepareStatement(query);
		ps.setInt(1, done);
		ps.setInt(2, function_id);
		ps.executeUpdate();
		ps.close();
	}
	
	
	/**
	 * {@inheritDoc}
	 */	
	public void updateFunctionOrder(int[] function_ids ) throws Exception  {
		String query="UPDATE r_function set disp_order=? where  id=?";
		PreparedStatement ps=this.connection().prepareStatement(query);		
		for(int i=0;i<function_ids.length;i++) {
			ps.setInt(1, i+1);
			ps.setInt(2, function_ids[i]);
			ps.executeUpdate();
		}
	}
	

	/**
	 * {@inheritDoc}
	 */	
	public void updateFunctionDeleted(int function_id) throws Exception  {
		String query="UPDATE r_function set deletedon=? where  id=?";
		PreparedStatement ps=this.connection().prepareStatement(query);
		ps.setTimestamp(1,new Timestamp(new Date().getTime()));
		ps.setInt(2, function_id);
		ps.executeUpdate();
	}
	
	
	/**
	 * {@inheritDoc}
	 */	
	public void updateFolderOrder(int[] folder_ids ) throws Exception  {
		String query="UPDATE folder set disp_order=? where  id=?";
		PreparedStatement ps=this.connection().prepareStatement(query);		
		for(int i=0;i<folder_ids.length;i++) {
			ps.setInt(1, i+1);
			ps.setInt(2, folder_ids[i]);
			ps.executeUpdate();
		}
	}
	
	
	/**
	 * {@inheritDoc}
	 */	
	public List listRFunctions() throws Exception {
		
		//following_user will be ignored as theme and notification implemented.
		StringBuffer sb = new StringBuffer();
		sb.append("SELECT a.*,IFNULL(TIMESTAMPDIFF(day,date(a.deletedon),now()),7) as deleteddays,b.folder_name,d.tagname as owner_tag,(select CONCAT(GROUP_CONCAT(cast(tags.tagname as CHAR(8000)) order by tags.tagname SEPARATOR ', '), ', ') from function_tags as st  left outer join tags on tags.id=st.tag_id WHERE st.function_id=a.id ) as stags ");
		sb.append(" FROM  r_function as a ");
		sb.append(" left outer join folder as b on a.folder_id=b.id ");
		sb.append(" left outer join function_group as c on c.group_uid=b.group_uid ");
		sb.append(" left outer join tags as d on d.id=a.owner_tag_id ");
		sb.append(" order by c.disp_order,b.disp_order,a.disp_order,a.function_name ");
		
		PreparedStatement ps=this.connection().prepareStatement(sb.toString());
		ResultSet rs=ps.executeQuery();
		Vector rtn=new Vector();
		while(rs.next()){			
			rtn.add(new BasicRowProcessor().toMap(rs));
		}
		return rtn;
	}
	
	
	/**
	 * {@inheritDoc}
	 */	
	public List listAllRFunctionNames() throws Exception {
		
		PreparedStatement ps=this.connection().prepareStatement("Select function_name  FROM  r_function ");
		
		ResultSet rs=ps.executeQuery();
		Vector rtn=new Vector();
		while(rs.next()){			
			rtn.add(rs.getString(1));
		}
		return rtn;
	}

	
	/**
	 * {@inheritDoc}
	 */	
	public List listAllRScriptNames(int folder_id) throws Exception {
		
		PreparedStatement ps=this.connection().prepareStatement("Select script_file FROM  r_function where folder_id=? order by disp_order");
		ps.setInt(1, folder_id);
		ResultSet rs=ps.executeQuery();
		Vector rtn=new Vector();
		while(rs.next()){			
			rtn.add(rs.getString(1));
		}
		return rtn;
	}
	
	
	/**
	 * {@inheritDoc}
	 */	
	public List autoCompleteFunctions(String keyword) throws Exception {
		
		 String theme=null;
		 
			// System.out.println(" legnth:"+keyword.split(":").length);
			 
			 if(keyword.contains(":") &&  keyword.split(":").length>1){
				 theme=keyword.split(":")[0];
				 keyword=keyword.split(":")[1];
			 }
			 
			 if(keyword.contains(":") &&  keyword.split(":").length==1){
				 theme=keyword.split(":")[0];
				 keyword=null;
				 
			 }
			 
		StringBuffer sb = new StringBuffer();	 
		sb.append("select a.id,a.function_name,c.group_name,b.folder_name,a.last_modified, ");
		sb.append(" (select CONCAT(GROUP_CONCAT(cast(tags.tagname as CHAR(8000)) order by tags.tagname SEPARATOR ', '), ', ') from function_tags as st  left outer join tags on tags.id=st.tag_id WHERE st.function_id=a.id ) as stags ");
		sb.append(" from r_function as a "); 
		sb.append(" left outer join folder as b on a.folder_id=b.id ");
		sb.append(" left outer join function_group as c on b.group_uid = c.group_uid ");
		sb.append(" where c.group_uid <> 'trash' ");
		
		if(keyword!=null){
			sb.append(" AND a.function_name like '%"+keyword+"%' ");
		 }
		 if(theme!=null){
			 sb.append("	 AND a.id in (select function_id from function_tags left outer join tags on function_tags.tag_id = tags.id where tags.tagname='thm-"+theme.trim()+"' ) " );
		 }	
		 sb.append(" order by c.disp_order, b.disp_order,a.disp_order ");
		
		PreparedStatement ps=this.connection().prepareStatement(sb.toString());
		ResultSet rs=ps.executeQuery();
		ArrayList rtn=new ArrayList();
		while(rs.next()){		
			rtn.add(new BasicRowProcessor().toMap(rs));
		}
		return rtn;
	}
	
	
	/**
	 * {@inheritDoc}
	 */	
	public List<Map> listOfFunctions(int folder_id) throws Exception {
		
		PreparedStatement ps=this.connection().prepareStatement("Select * FROM  r_function where folder_id=? order by disp_order");
		ps.setInt(1, folder_id);
		ResultSet rs=ps.executeQuery();
		ArrayList rtn=new ArrayList();
		while(rs.next()){		
			rtn.add(new BasicRowProcessor().toMap(rs));
		}
		return rtn;
	}

	
	/**
	 * {@inheritDoc}
	 */	
	public Map getRFunction(int function_id) throws Exception {
		
		StringBuffer sb = new StringBuffer();	
		sb.append("SELECT a.*,b.folder_name,IFNULL(TIMESTAMPDIFF(day,date(a.deletedon),now()),7) as deleteddays,b.folder_name,c.tagname as owner_tag,(select CONCAT(GROUP_CONCAT(cast(tags.tagname as CHAR(8000)) order by tags.tagname SEPARATOR ', '), ', ') from function_tags as st  left outer join tags on tags.id=st.tag_id WHERE st.function_id=a.id ) as stags FROM  r_function as a ");		
		sb.append(" left outer join tags as c on c.id=a.owner_tag_id ");
		sb.append(" left outer join folder as b on a.folder_id=b.id ");
		sb.append(" WHERE  a.id=? ORDER BY a.function_name ");
		
		PreparedStatement ps=this.connection().prepareStatement(sb.toString());
		 
		ps.setInt(1, function_id);
		ResultSet rs=ps.executeQuery();
		
		Map rtn=null;
		if(rs.next()){			
			rtn=new BasicRowProcessor().toMap(rs);
		}
		return rtn;
	}

	
	/**
	 * {@inheritDoc}
	 */	
	public Map getRFunctionForScriptFile(String scriptfile) throws Exception {
		
		StringBuffer sb = new StringBuffer();	
		sb.append("Select a.*,b.folder_name, ");
		sb.append("(select CONCAT(GROUP_CONCAT(cast(tags.tagname as CHAR(8000)) order by tags.tagname SEPARATOR ', '), ', ') from function_tags as st  left outer join tags on tags.id=st.tag_id WHERE st.function_id=a.id ) as stags ");
		sb.append("FROM  r_function as a ");
		sb.append("left outer join folder as b on a.folder_id=b.id ");
		sb.append("WHERE  a.script_file=? ");
		sb.append("ORDER BY a.function_name ");
		PreparedStatement ps=this.connection().prepareStatement(sb.toString());
		ps.setString(1, scriptfile);
		ResultSet rs=ps.executeQuery();		
		Map rtn=null;
		if(rs.next()){			
			rtn=new BasicRowProcessor().toMap(rs);
		}
		return rtn;
	}

	
	/**
	 * {@inheritDoc}
	 */		
	public Vector getRFunctions(Vector functionnames) throws Exception {
		
		//PreparedStatement ps=this.connection().prepareStatement("Select *  FROM  r_function WHERE  id=?");
		
		String sfunc="''";
		for(Iterator i=functionnames.iterator();i.hasNext();){
			sfunc+=",'"+i.next()+"'";
		}
		
		StringBuffer sb = new StringBuffer();	
		sb.append("Select a.*, ");
		sb.append("IFNULL(TIMESTAMPDIFF(day,date(a.deletedon),now()),7) as deleteddays, ");
		sb.append("b.folder_name, ");
		sb.append("(select CONCAT(GROUP_CONCAT(cast(tags.tagname as CHAR(8000)) order by tags.tagname SEPARATOR ', '), ', ') from function_tags as st  left outer join tags on tags.id=st.tag_id WHERE st.function_id=a.id ) as stags ");
		sb.append("FROM  r_function as a left outer join folder as b on a.folder_id=b.id WHERE  a.function_name IN ("+sfunc+") ORDER BY a.function_name ");
		PreparedStatement ps=this.connection().prepareStatement(sb.toString());
 

		ResultSet rs=ps.executeQuery();
		
		Vector rtn=new Vector();
		while(rs.next()){			
			rtn.add(new BasicRowProcessor().toMap(rs));
		}
		return rtn;
	}


	/**
	 * {@inheritDoc}
	 */		
	public List listFunctionGroups() throws Exception {
		
		PreparedStatement ps=this.connection().prepareStatement("Select * from function_group ORDER BY disp_order");
		
		ResultSet rs=ps.executeQuery();
		Vector rtn=new Vector();
		while(rs.next()){			
			rtn.add(new BasicRowProcessor().toMap(rs));
		}
		return rtn;
	}

	
	/**
	 * {@inheritDoc}
	 */		
	public Map getRFunction(String function_name) throws Exception {
		
		PreparedStatement ps=this.connection().prepareStatement("Select *  FROM  r_function WHERE  function_name=?");
		ps.setString(1, function_name);
		ResultSet rs=ps.executeQuery();		
		Map rtn=null;
		if(rs.next()){			
			rtn=new BasicRowProcessor().toMap(rs);
		}
		return rtn;
	}
	
	
	/**
	 * {@inheritDoc}
	 */		
	public int createFunction(int folder_id,String functionname, String path,int func_type) throws Exception {
		PreparedStatement ps=this.connection().prepareStatement("INSERT INTO r_function(folder_id,function_name,script_file,last_modified,is_class) VALUES(?,?,?,?,?)",Statement.RETURN_GENERATED_KEYS);
		ps.setInt(1,folder_id);
		ps.setString(2, functionname);
		ps.setString(3, path);		
		ps.setTimestamp(4,new Timestamp(new Date().getTime()));
		ps.setInt(5, func_type);
		ps.executeUpdate();
		int id=0;
		ResultSet generatedKeys = ps.getGeneratedKeys();
		
        if (generatedKeys.next()) {
            id=generatedKeys.getInt(1);
        }		
		ps.close();	

		return id; 
	}
	
	
	/**
	 * {@inheritDoc}
	 */		
	public void renameFunction(int function_id, String functionname, String script_file) throws Exception {
		PreparedStatement ps=this.connection().prepareStatement("UPDATE r_function SET function_name=?,script_file=?,last_modified=? WHERE id=?");		
		ps.setString(1, functionname);
		ps.setString(2, script_file);
		ps.setTimestamp(3,new Timestamp(new Date().getTime()));
		ps.setInt(4,function_id);
				
		ps.executeUpdate();
		
	}
	
	
	/**
	 * {@inheritDoc}
	 */		
	public void updatedOwnerIDNow(int owner_id,int function_id) throws Exception {
		PreparedStatement ps=this.connection().prepareStatement("UPDATE r_function SET last_modified=?,owner_tag_id=? WHERE id=?");		
		ps.setTimestamp(1,new Timestamp(new Date().getTime()));
		ps.setInt(2,owner_id);
		ps.setInt(3,function_id);
		ps.executeUpdate();
	}
	
	
	/**
	 * {@inheritDoc}
	 */		
	public void deleteFunction(int function_id) throws Exception {
		PreparedStatement ps=this.connection().prepareStatement("DELETE FROM r_function  WHERE id=?");
		ps.setInt(1,function_id);
		ps.executeUpdate();
		
		PreparedStatement ps2=this.connection().prepareStatement("DELETE FROM function_tags  WHERE function_id=?");
		ps2.setInt(1,function_id);
		ps2.executeUpdate();
	}
	
	
	/**
	 * {@inheritDoc}
	 */		
	public void updateFunctionFolder(int function_id,int new_folder_id) throws Exception {
		PreparedStatement ps=this.connection().prepareStatement("UPDATE r_function SET folder_id=? WHERE id=?");
		
		ps.setInt(1,new_folder_id);
		ps.setInt(2, function_id);
		ps.executeUpdate();
		ps.close();
		
		StringBuffer sb = new StringBuffer(); 
		sb.append("update r_function set disp_order = ( ");
		sb.append("select max_disp_order from ( ");
		sb.append("select max(disp_order)+1 as max_disp_order from r_function where folder_id = ? "); 
		sb.append(") as temp_table ");
		sb.append(") where id = ? ");
				
		PreparedStatement ps1=this.connection().prepareStatement(sb.toString());
		ps1.setInt(1,new_folder_id);
		ps1.setInt(2, function_id);
		ps1.executeUpdate();
		ps1.close();		
 
	}
	
	
	/**
	 * {@inheritDoc}
	 */		
	public void updateLock(int function_id,String lockedby) throws Exception {
		PreparedStatement ps=this.connection().prepareStatement("UPDATE r_function SET lockedby=? WHERE id=?");
		ps.setString(1,lockedby);
		ps.setInt(2, function_id);
		ps.executeUpdate();
		ps.close();
	}
	
	
	/**
	 * {@inheritDoc}
	 */		
	public List listOfFolders( ) throws Exception {
		
		PreparedStatement ps=null;		 
		//ps=this.connection().prepareStatement("Select a.* FROM folder as a left outer join function_group as b on b.group_uid = a.group_uid ORDER BY b.disp_order,a.disp_order ");
		
		StringBuffer sb = new StringBuffer();
		sb.append("Select a.*, ");
		sb.append("(select CONCAT(GROUP_CONCAT(cast(tags.tagname as CHAR(8000)) order by tags.tagname SEPARATOR ', '), ', ') from folder_tags as st  left outer join tags on tags.id=st.tag_id WHERE st.folder_id=a.id ) as stags ");
		sb.append("FROM folder as a left outer join function_group as b on b.group_uid = a.group_uid ORDER BY b.disp_order,a.disp_order ");
		ps=this.connection().prepareStatement(sb.toString());		
		
		ResultSet rs=ps.executeQuery();
		Vector rtn=new Vector();
		while(rs.next()){			
			rtn.add(new BasicRowProcessor().toMap(rs));
		}
		return rtn;
	}

	
	/**
	 * {@inheritDoc}
	 */		
	public List listOfFolders(String group_id) throws Exception {
		
		PreparedStatement ps=null;		
		
		StringBuffer sb = new StringBuffer();
		sb.append("Select *, ");
		sb.append("(select CONCAT(GROUP_CONCAT(cast(tags.tagname as CHAR(8000)) order by tags.tagname SEPARATOR ', '), ', ') from folder_tags as st  left outer join tags on tags.id=st.tag_id WHERE st.folder_id=a.id ) as stags ");
		sb.append("FROM folder as a WHERE group_uid=?  ORDER BY disp_order ");
		ps=this.connection().prepareStatement(sb.toString());		 
		ps.setString(1, group_id);
		ResultSet rs=ps.executeQuery();
		Vector rtn=new Vector();
		while(rs.next()){			
			rtn.add(new BasicRowProcessor().toMap(rs));
		}
		return rtn;
	}

	
	/**
	 * {@inheritDoc}
	 */		
	public void moveFolder(int folder_id, String new_group_id) throws Exception{
		
		PreparedStatement ps=this.connection().prepareStatement("UPDATE folder SET group_uid = ? WHERE  id=?");
		ps.setString(1, new_group_id);
		ps.setInt(2,folder_id);
		ps.executeUpdate();
		ps.close();
	}
	
	 
	/**
	 * {@inheritDoc}
	 */		
	public int getFolderID(String folder_name) throws Exception {
		
		PreparedStatement ps=null;		 
		ps=this.connection().prepareStatement("Select * FROM folder WHERE folder_name=?");
		ps.setString(1, folder_name);
		ResultSet rs=ps.executeQuery();
		int rtn=0;
		if(rs.next()){			
		 rtn=rs.getInt("id");
		}
		return rtn;
	}
	
	
	/**
	 * {@inheritDoc}
	 */		
	public int createFolder(String folder, String new_group_id) throws Exception{		
		
		PreparedStatement ps=this.connection().prepareStatement("INSERT INTO folder(folder_name,group_uid) VALUES(?,?)",Statement.RETURN_GENERATED_KEYS);
		ps.setString(1, folder);
		ps.setString(2, new_group_id);				 
		ps.executeUpdate();
		ResultSet generatedKeys = ps.getGeneratedKeys();
		int id=0;
	    if (generatedKeys.next()) {
	    	id=generatedKeys.getInt(1);
	    }	    
		ps.close();
		return id; 
	}

	
	/**
	 * {@inheritDoc}
	 */		
	public void renameFolder(int folder_id,String foldername) throws Exception {
		PreparedStatement ps=this.connection().prepareStatement("UPDATE folder SET folder_name = ? WHERE  id=?");
		ps.setString(1, foldername);
		ps.setInt(2,folder_id);
		ps.executeUpdate();
		ps.close();

	}

	
	/**
	 * {@inheritDoc}
	 */		
	public void removeFolder(int folder_id) throws Exception {
		PreparedStatement ps=this.connection().prepareStatement("DELETE FROM folder WHERE  id=?");
		ps.setInt(1,folder_id);
		ps.executeUpdate();
		ps.close();
	}
	
	
	/**
	 * {@inheritDoc}
	 */		
	 public Vector getTags() throws Exception {
		 Vector rtn=new Vector();
		 PreparedStatement ps=this.connection().prepareStatement("select * FROM tags ORDER BY tagname");			
		 ResultSet rs=ps.executeQuery();
		 while(rs.next()){				
				rtn.add(new BasicRowProcessor().toMap(rs));
		 }
		 return rtn;
	 }
	 
	 
	/**
	 * {@inheritDoc}
	 */		
	 public Vector getTagIds4Function(int function_id) throws Exception {
		 PreparedStatement ps=this.connection().prepareStatement("select tag_id FROM function_tags WHERE function_id=?");
		 ps.setInt(1, function_id);
		 ResultSet rs=ps.executeQuery();
		 Vector rtn=new Vector();
		 while(rs.next()){			 
				rtn.add(rs.getInt("tag_id"));
		 }
		 return rtn;
	 }
	
	 
	/**
	 * {@inheritDoc}
	 */		
	 public ArrayList getTagIds4Folder(int folder_id, String tablename) throws Exception {
		 PreparedStatement ps=this.connection().prepareStatement("select tag_id FROM "+tablename+" WHERE folder_id=?");
		 ps.setInt(1, folder_id);
		 ResultSet rs=ps.executeQuery();
		 ArrayList rtn=new ArrayList();
		 while(rs.next()){			 
				rtn.add(rs.getInt("tag_id"));
		 }
		 return rtn;
	 }
	
 
	/**
	 * {@inheritDoc}
	 */		
	 public List getThemeTags4Folder(int folder_id) throws Exception {
		 
		 String q="select replace(b.tagname,'thm-','') as theme FROM folder_tags as a ";
		 q+="left outer join "+getTagTable()+" as b on a.tag_id=b.id ";
		 q+="where a.folder_id=? ";
		 q+="and left(b.tagname,4)='thm-' ";

		 PreparedStatement ps=this.connection().prepareStatement(q);
		 ps.setInt(1, folder_id);
		 
		 ResultSet rs=ps.executeQuery();
		 ArrayList rtn=new ArrayList();
		 while(rs.next()){			 
				rtn.add(rs.getString("theme"));
		 }
		 return rtn;
	 }

 
	/**
	 * {@inheritDoc}
	 */		 
	 public void updateTagIds4Folder(int folder_id, List tag_ids,int removeExisting, String tablename) throws Exception {
		 if(SchedulerDB.REMOVE_BEFORE_UPDATE==removeExisting){
			 PreparedStatement ps=this.connection().prepareStatement("DELETE FROM "+tablename+" WHERE folder_id=?");
			 //ps.setString(1, tablename);
			 ps.setInt(1, folder_id);
			 ps.executeUpdate();
			 ps.close();
		 }
		 PreparedStatement ps1=this.connection().prepareStatement("INSERT INTO "+tablename+"(folder_id,tag_id) VALUES(?,?)");
		 for(Iterator i=tag_ids.iterator();i.hasNext();){
			 //ps1.setString(1, tablename);
			 ps1.setInt(1, folder_id);
			 ps1.setInt(2, Integer.parseInt((String)i.next()));
			 ps1.executeUpdate();
		 }

		 ps1.close();
	 }


	/**
	 * {@inheritDoc}
	 */		
	 public void updateLast2UsersTag(int function_id, int user_tagid) throws Exception {
		 
		 // works without 'limit 2', the result is equal.
		 String query="DELETE FROM function_tags WHERE id IN ( select id from ( SELECT st.id FROM function_tags as st ";
		 query+=" LEFT OUTER JOIN tags  on tags.id=st.tag_id ";
		 query+=" WHERE st.id NOT IN( ";
		 query+="   select st1.id from function_tags as st1 ";
		 query+="   left outer join tags as t1 on t1.id=st1.tag_id ";
		 query+="   WHERE LEFT(t1.tagname,4)='usr-' AND st1.function_id=st.function_id ";
		 query+="   order by st1.id desc "; 
		 query+=" ) "; 
		 query+=" AND LEFT(tags.tagname,4)='usr-' AND st.function_id=? ) temp_table) ";
	     PreparedStatement ps=this.connection().prepareStatement(query);
		 ps.setInt(1, function_id);
		 ps.executeUpdate();
		 ps.close();
		 
		 PreparedStatement ps2=this.connection().prepareStatement("DELETE FROM function_tags WHERE function_id=? AND tag_id=?");
		 ps2.setInt(1, function_id);
		 ps2.setInt(2, user_tagid);
		 ps2.executeUpdate();
		 
		 
		 PreparedStatement ps1=this.connection().prepareStatement("INSERT INTO function_tags(function_id,tag_id) VALUES(?,?)");
		 ps1.setInt(1, function_id);
		 ps1.setInt(2, user_tagid);
		 ps1.executeUpdate();
		 ps1.close();
		 
	 }
	 
	 
	/**
	 * {@inheritDoc}
	 */		 
	 public int addIfTagNotExist(String tagname) throws Exception {
		 
		    PreparedStatement ps=this.connection().prepareStatement("select * FROM tags WHERE tagname=?");
		    ps.setString(1, tagname);
		    ResultSet rs=ps.executeQuery();
		    int id=0;
		    if(rs.next()){
		    	id=rs.getInt("id");
		    }else{		 
		    	PreparedStatement ps1=this.connection().prepareStatement("insert into tags(tagname) values(?)",Statement.RETURN_GENERATED_KEYS);
			    ps1.setString(1, tagname);			 
			    ps1.executeUpdate();
			    ResultSet generatedKeys = ps1.getGeneratedKeys();			
			    if (generatedKeys.next()) {
			    	id=generatedKeys.getInt(1);
			    }
		    }
		   return id;
	 }

	 
	/**
	 * {@inheritDoc}
	 */		 
	 public void addTagIds4Function(int function_id, int tag_id) throws Exception {
		  
		 PreparedStatement ps=this.connection().prepareStatement("DELETE FROM function_tags WHERE function_id=? and tag_id=?");
		 ps.setInt(1, function_id);
		 ps.setInt(2, tag_id);
		 ps.executeUpdate();
		 ps.close();
		 
		 PreparedStatement ps1=this.connection().prepareStatement("INSERT INTO function_tags(function_id,tag_id) VALUES(?,?)");
		 ps1.setInt(1, function_id);
	     ps1.setInt(2, tag_id);
		 ps1.executeUpdate();
		 ps1.close();
	 }
	 
	 
	/**
	 * {@inheritDoc}
	 */		 
	 public void removeTagIds4Function(int function_id, int tag_id) throws Exception {
		 PreparedStatement ps=this.connection().prepareStatement("DELETE FROM function_tags WHERE function_id=? and tag_id=?");
		 ps.setInt(1, function_id);
		 ps.setInt(2, tag_id);
		 ps.executeUpdate();
		 ps.close();
	 }
	
	 
	/**
	 * {@inheritDoc}
	 */		
	public void setGroupOrder( Vector groupids) throws Exception {		
			 
			PreparedStatement ps1=this.connection().prepareStatement("UPDATE function_group SET disp_order=? WHERE group_uid=?");
			int count=0;
			for(Iterator<String> i=groupids.iterator();i.hasNext();){
				ps1.setInt(1, count++);
				ps1.setString(2, i.next());			
				ps1.executeUpdate();
			}
	}

	
	/**
	 * {@inheritDoc}
	 */		
	public Map getAllFunctionNamesID(String wherecond) throws Exception {
		
		String query="select id,function_name from r_function ";
		if(wherecond!=null && !wherecond.equals("")) {
			query+=" WHERE "+wherecond;
		}
		PreparedStatement ps=this.connection().prepareStatement(query);
		ResultSet rs=ps.executeQuery();
		HashMap h=new HashMap();
		while(rs.next()){
			int id=rs.getInt("id");
			String fname=rs.getString("function_name");
			h.put(id, fname);
		}
		return h;
	}
	
	
	/**
	 * {@inheritDoc}
	 */				
	public boolean updatePeerPackage(String peername, ArrayList<PeerPackage>plist) throws Exception {
		
		try{
			this.connection().setAutoCommit(false);
			PreparedStatement ps=this.connection().prepareStatement("DELETE FROM peer_packages WHERE peername=?");
			ps.setString(1, peername);
			ps.executeUpdate();
			ps.close();
			PreparedStatement ps1=this.connection().prepareStatement("INSERT INTO peer_packages(peername,packagename,version,lastchecked) VALUES(?,?,?,?)");			
			for(PeerPackage pp: plist){
				ps1.setString(1, peername);
				ps1.setString(2, pp.getPackagename());
				ps1.setString(3, pp.getVersion());
				ps1.setTimestamp(4, new Timestamp(pp.getLastchecked().getTime()));
				ps1.executeUpdate();
			}
			ps1.close();			
			this.connection().commit();
			return true;
		}catch(Exception e){
			this.connection().rollback();
			throw e;
		}finally{
			this.connection().setAutoCommit(false);
		}
	}
	
	
	/**
	 * {@inheritDoc}
	 */		
	public Map getPackageInfo(String pname) throws Exception {
		
		StringBuffer sb = new StringBuffer();
		sb.append("select a.version, ");
		sb.append("(select CONCAT(GROUP_CONCAT(cast(b.peername as CHAR(8000)) SEPARATOR ', '), ', ') from peer_packages as b where b.packagename=? and b.version=a.version ) as peers ");
		sb.append("from peer_packages as a ");
		sb.append("where a.packagename=? ");
		sb.append("group by a.version ");
		PreparedStatement ps1=this.connection().prepareStatement(sb.toString());
		ps1.setString(1,pname);
		ps1.setString(2,pname+"");
		ResultSet rs=ps1.executeQuery();
		LinkedHashMap<String, String> res=new LinkedHashMap<String, String>();
		while(rs.next()){
			String version=rs.getString("version");
			String peers=rs.getString("peers");
			peers=peers.substring(0,peers.lastIndexOf(","));			
			res.put(version, peers);
		}
		rs.close();
		ps1.close();
		
		return res;

		//return null;
		
	}
	
	
	/**
	 * {@inheritDoc}
	 */		
	public List getDefaultHierarchyDependsIds(String pname) throws Exception {
		/*
		String query="";
		query+="select a.* from folder as a ";
		query+="left outer join function_group as b on a.group_uid = b.group_uid ";
		query+="where b.disp_order<=( ";
		query+="  select function_group.disp_order from folder "; 
		query+="  left outer join function_group  on folder.group_uid = function_group.group_uid "; 
		query+="  where folder_name=? ";
		query+=") ";
		query+="order by b.disp_order,a.disp_order ";
		*/
		
		String query="";
		query+="select a.id,a.folder_name from folder as a "; 
		query+=" left outer join function_group as b on a.group_uid = b.group_uid ";
		query+=" order by b.disp_order, a.disp_order  ";		
		PreparedStatement ps1=this.connection().prepareStatement(query);
		//ps1.setString(1,pname);
		ResultSet rs=ps1.executeQuery();
		//LinkedHashMap<Integer, String> res=new LinkedHashMap<Integer, String>();
		ArrayList res=new ArrayList();
		boolean cont=true;
		while(rs.next() && cont){
			Integer id=rs.getInt("id");
			String packname=rs.getString("folder_name");
			if(packname.equals(pname)){
				cont=false;
			}else{
				res.add(id);	
			}
		}
		return res;
	}
	
	
	/**
	 * {@inheritDoc}
	 */		
	public List listRelatedFolderIds(int parent_folder) throws Exception {
		String query="select related_folder_id from folder_dependencies where folder_id=?" ;
		PreparedStatement ps1=this.connection().prepareStatement(query);
		ps1.setInt(1,parent_folder);
		ResultSet rs=ps1.executeQuery();
		ArrayList res=new ArrayList();
		while(rs.next()){
			Integer id=rs.getInt("related_folder_id");			 
			res.add(id);
		}
		return res;
	}
	
	
	/**
	 * {@inheritDoc}
	 */		
	public void updateRelatedFolderIds(int folder_id, List<Integer> ids) throws Exception {
		PreparedStatement ps1=this.connection().prepareStatement("DELETE from folder_dependencies where folder_id=?");
		ps1.setInt(1, folder_id);
		ps1.executeUpdate();
		ps1.close();
		
		PreparedStatement ps=this.connection().prepareStatement("INSERT INTO folder_dependencies(folder_id,related_folder_id) VALUES(?,?)");
		for(Integer i:ids){
			ps.setInt(1, folder_id);
			ps.setInt(2, i);
			ps.executeUpdate();
		}
		ps.close();
		
	}
	
	
	/**
	 * {@inheritDoc}
	 */		
	public void addTag4Package(int tag_id,int function_ids[],boolean make_owner_also) throws Exception {
		
		try{
			
			 this.connection().setAutoCommit(false);
			 PreparedStatement ps=this.connection().prepareStatement("DELETE FROM function_tags WHERE function_id=? and tag_id=?");
			 PreparedStatement ps1=this.connection().prepareStatement("insert into function_tags(function_id,tag_id)  VALUES(?,?)");
			 PreparedStatement ps2=this.connection().prepareStatement("UPDATE r_function SET owner_tag_id=? WHERE id=?");
			 for(int i=0;i<function_ids.length;i++){
				
				 ps.setInt(1, function_ids[i]);
				 ps.setInt(2, tag_id);
				 ps.executeUpdate();
				 
				 
				 log.debug("tags deleted");
				 
			
				 ps1.setInt(1, function_ids[i]);
				 ps1.setInt(2, tag_id);  
				 ps1.executeUpdate();
			
				 
				 log.debug("tags added");
				 
				 if(make_owner_also){					
					 ps2.setInt(1, tag_id);	
					 ps2.setInt(2, function_ids[i]);	
					 ps2.executeUpdate();					 
				 }
			 }
			 ps.close();
			 ps1.close();
			 ps2.close();
			 
			 /*
			 PreparedStatement ps=this.connection().prepareStatement("DELETE FROM function_tags WHERE function_id in(select id from r_function where folder_id=?) and tag_id=?");
			 ps.setInt(1, folder_id);
			 ps.setInt(2, tag_id);
			 ps.executeUpdate();
			 ps.close();
			 
			 log.debug("tags deleted");
			 
			 PreparedStatement ps1=this.connection().prepareStatement("insert into function_tags(function_id,tag_id) select id,"+tag_id+"  from r_function where folder_id=?");
			 ps1.setInt(1, folder_id);		  
			 ps1.executeUpdate();
			 ps1.close();
			 
			 log.debug("tags added");
			 
			 if(make_owner_also){
				 PreparedStatement ps2=this.connection().prepareStatement("UPDATE r_function SET owner_tag_id=? WHERE folder_id=?");
				 ps2.setInt(1, tag_id);	
				 ps2.setInt(2, folder_id);	
				 ps2.executeUpdate();	 
			 }
			 */
			 this.connection().commit();
			    
		}catch(Exception e){
			this.connection().rollback();
			throw e;
		}finally{
			this.connection().setAutoCommit(true);
		}
	}
	
	
	/**
	 * {@inheritDoc}
	 */		
	public void removeTag4Package(int tag_id,int function_ids[]) throws Exception {
		
		try{
			this.connection().setAutoCommit(false);
			
			 this.connection().setAutoCommit(false);
			 for(int i=0;i<function_ids.length;i++){
				 PreparedStatement ps=this.connection().prepareStatement("DELETE FROM function_tags WHERE function_id=? and tag_id=?");
				 ps.setInt(1, function_ids[i]);
				 ps.setInt(2, tag_id);
				 ps.executeUpdate();
				 ps.close();
				 
				 log.debug("tags deleted");
				 
				 PreparedStatement ps2=this.connection().prepareStatement("update r_function set owner_tag_id=(select tag_id from function_tags where function_id=r_function.id order by id limit 1) where id=?");
				 ps2.setInt(1, function_ids[i]);
				 ps2.executeUpdate();
				 ps2.close();
			 
			 }
			 
			this.connection().commit();   
		}catch(Exception e){
			this.connection().rollback();
			throw e;
		}finally{
			this.connection().setAutoCommit(true);
		}
	}

	
	/**
	 * {@inheritDoc}
	 */		
	public void addNotificationTag4Package(int tag_id,int function_ids[],boolean add_also) throws Exception {
		
		try{
			
			 this.connection().setAutoCommit(false);
			 PreparedStatement ps=this.connection().prepareStatement("DELETE FROM function_followtags WHERE function_id=? and tag_id=?");
			 PreparedStatement ps1=this.connection().prepareStatement("insert into function_followtags(function_id,tag_id)  VALUES(?,?)");
			 
			 for(int i=0;i<function_ids.length;i++){
				
				 ps.setInt(1, function_ids[i]);
				 ps.setInt(2, tag_id);
				 ps.executeUpdate();
				 
				 if(add_also){
					ps1.setInt(1, function_ids[i]);
				 	ps1.setInt(2, tag_id);  
				 	ps1.executeUpdate();
				 }
			 
			 }
			 ps.close();
			 ps1.close();
			 
			 /*
			 PreparedStatement ps=this.connection().prepareStatement("DELETE FROM function_tags WHERE function_id in(select id from r_function where folder_id=?) and tag_id=?");
			 ps.setInt(1, folder_id);
			 ps.setInt(2, tag_id);
			 ps.executeUpdate();
			 ps.close();
			 
			 log.debug("tags deleted");
			 
			 PreparedStatement ps1=this.connection().prepareStatement("insert into function_tags(function_id,tag_id) select id,"+tag_id+"  from r_function where folder_id=?");
			 ps1.setInt(1, folder_id);		  
			 ps1.executeUpdate();
			 ps1.close();
			 
			 log.debug("tags added");
			 
			 if(make_owner_also){
				 PreparedStatement ps2=this.connection().prepareStatement("UPDATE r_function SET owner_tag_id=? WHERE folder_id=?");
				 ps2.setInt(1, tag_id);	
				 ps2.setInt(2, folder_id);	
				 ps2.executeUpdate();	 
			 }
			 */
			 this.connection().commit();
			    
		}catch(Exception e){
			this.connection().rollback();
			throw e;
		}finally{
			this.connection().setAutoCommit(true);
		}
	}
	
	
	/**
	 * {@inheritDoc}
	 */		
	public String getFolderThemeByRFunctionId(int r_function_id) throws Exception {
		String result = null;
		PreparedStatement ps=this.connection().prepareStatement("select replace(tagname,'thm-','') as theme from tags where id = (select b.tag_id from r_function a inner join folder_tags b on a.folder_id=b.folder_id where a.id = ?)");
		ps.setInt(1, r_function_id);		
		ResultSet rs=ps.executeQuery();
		while(rs.next()){
			result = rs.getString("theme");
		}
		ps.close();
		rs.clearWarnings();		
		return result;
	}	


	public void syncLib(int counter, int theme, int wikiDone, String rScript) throws Exception {
		String query =  "INSERT INTO r_function(function_name, folder_id, script_file, is_wiki_done, last_modified, owner_tag_id, is_class) " +
		   		"VALUES('"+rScript+"',"+theme+",'"+rScript+".r',"+wikiDone+",NOW(),0,0)";
		PreparedStatement ps=this.connection().prepareStatement(query);
		ps.executeUpdate();
		ps.close();

		/*int tag = (theme == 1 ? 1 : theme == 2 ? 14 : theme == 3 ? 7 : 3);

		String query2 = "INSERT INTO function_tags(function_id, tag_id) VALUES ("+counter+", "+tag+")";
		PreparedStatement ps2=this.connection().prepareStatement(query2);
		ps2.executeUpdate();
		ps2.close();*/

	} 

	public void truncateFunctions() throws Exception {
		String query = "TRUNCATE table r_function";
		PreparedStatement ps=this.connection().prepareStatement(query);
		ps.executeUpdate();
		ps.close();		

		String query2 = "TRUNCATE table function_tags";
		PreparedStatement ps2=this.connection().prepareStatement(query2);
		ps2.executeUpdate();
		ps2.close();
	}
	
}
