/******************************************************************************
*
* Copyright: Intellectual Property of Four Elements Capital Pte Ltd, Singapore.
* All rights reserved.
*
******************************************************************************/

package com.fourelementscapital.db.mariadb;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.Statement;
import java.util.LinkedHashMap;
import java.util.Map;
import java.util.TreeMap;
import java.util.Vector;

import org.apache.logging.log4j.Logger;
import org.apache.logging.log4j.LogManager;

import com.fourelementscapital.db.GeneralUtilDB;
import com.fourelementscapital.db.ReferenceDB;
import com.fourelementscapital.db.vo.CommodityInfo;

/**
 * Reference MariaDB DAO implementation
 */	
public class ReferenceDBMariaDB extends ReferenceDB {
	
private Logger log=LogManager.getLogger(ReferenceDB.class.getName());
	
	/**
	 * {@inheritDoc}
	 */	
	public Map getComContracts() throws Exception {
		
		Statement st=this.connection().createStatement();
		//ResultSet rs=st.executeQuery("select left(ltrim(name),2) as com,name  from contract_info order by name,last_trade_date");
		ResultSet rs=st.executeQuery("select ifnull(commodity_specify, left(ltrim(name),2)) as com,name  from contract_info order by name,last_trade_date");
		
		Vector commodity=new Vector();
		TreeMap contract=new TreeMap();
		while(rs.next()){
			String com=rs.getString("com");
			String cname=rs.getString("name");
			Vector t=null;
			if(commodity.contains(com)){
				t=(Vector)contract.get(com);
			}else{				
				t=new Vector();
				contract.put(com,t);
				commodity.add(com);
			}
			t.add(cname);
		}
		st.close();
		rs.close();
		TreeMap rtn=new TreeMap();
		rtn.put("commodity", commodity);
		rtn.put("contract", contract);
		return rtn;
	}
	

	/**
	 * {@inheritDoc}
	 */	
	public void deleteFieldMapping(int id) throws Exception {
		
		PreparedStatement ps=this.connection().prepareStatement("DELETE FROM static_field_mapping WHERE id=?");
		ps.setInt(1, id);
		ps.executeUpdate();
		ps.close();
	}
	
	
	/**
	 * {@inheritDoc}
	 */	
	public void addFieldMapping(String dbfield, String bbfield) throws Exception {
		
		PreparedStatement ps=this.connection().prepareStatement("INSERT INTO static_field_mapping(friendly_name, bb_fieldname) VALUES (?,?)");
		ps.setString(1, dbfield);
		ps.setString(2, bbfield);
		ps.executeUpdate();
		ps.close();
	}
	
	
	/**
	 * {@inheritDoc}
	 */	
	public Vector getFieldMapping() throws Exception {
		
		PreparedStatement ps=this.connection().prepareStatement("Select * from static_field_mapping");
		
		ResultSet rs=ps.executeQuery();
		Vector rtn=new Vector();
		while(rs.next()){			
			//rtn.add(new BasicRowProcessor().toMap(rs));
			rtn.add(GeneralUtilDB.resultsetToMap(rs));
		}
		return rtn;
	}


	/**
	 * {@inheritDoc}
	 */	
	public void deleteSecFieldMapping(int id) throws Exception {
		
		PreparedStatement ps=this.connection().prepareStatement("DELETE FROM security_static_fieldmapping WHERE id=?");
		ps.setInt(1, id);
		ps.executeUpdate();
		ps.close();
	}
	
	
	/**
	 * {@inheritDoc}
	 */	
	public void addSecFieldMapping(String dbfield, String bbfield) throws Exception {
		
		PreparedStatement ps=this.connection().prepareStatement("INSERT INTO security_static_fieldmapping(friendly_name, bb_fieldname) VALUES (?,?)");
		ps.setString(1, dbfield);
		ps.setString(2, bbfield);
		ps.executeUpdate();
		ps.close();
	}
	
	
	/**
	 * {@inheritDoc}
	 */	
	public Vector getSecFieldMapping() throws Exception {
		
		PreparedStatement ps=this.connection().prepareStatement("Select * from security_static_fieldmapping");
		
		ResultSet rs=ps.executeQuery();
		Vector rtn=new Vector();
		while(rs.next()){			
			//rtn.add(new BasicRowProcessor().toMap(rs));
			rtn.add(GeneralUtilDB.resultsetToMap(rs));
		}
		return rtn;
	}
	
	
	/**
	 * {@inheritDoc}
	 */	
	public void updateCommodity(CommodityInfo comm) throws Exception {
		
		PreparedStatement ps=this.connection().prepareStatement("SELECT  * from commodity_info WHERE commodity=?");
		ps.setString(1, comm.getCommodity());	
		ResultSet rs=ps.executeQuery();
		String query="INSERT INTO commodity_info(bbticker,fepricesample,fetradeunit,glsample,glticker,multiplefactorbn,name,nepricesample,netradeunit,nename,sector,units,multiplefactor,commodity) values(?,?,?,?,?,?,?,?,?,?,?,?,?,?)";
		if(rs.next()){
			query="UPDATE commodity_info SET bbticker=?,fepricesample=?,fetradeunit=?,glsample=?,glticker=?,multiplefactorbn=?,name=?,nepricesample=?,netradeunit=?,nename=?,sector=?,units=?,multiplefactor=? WHERE commodity=?";
		}
		ps.close();
		PreparedStatement ps1=this.connection().prepareStatement(query);
		ps1.setString(1,comm.getBbticker());
		ps1.setString(2,comm.getFepricesample());
		ps1.setString(3,comm.getFetradeunit());
		ps1.setString(4,comm.getGlsample());
		ps1.setString(5,comm.getGlticker());
		ps1.setString(6,comm.getMultiplefactorbn());
		ps1.setString(7,comm.getName());
		ps1.setString(8,comm.getNepricesample());
		ps1.setString(9,comm.getNetradeunit());
		ps1.setString(10,comm.getNename());
		ps1.setString(11,comm.getSector());
		ps1.setString(12,comm.getUnits());
		ps1.setString(13,comm.getMultiplefactor());
		ps1.setString(14,comm.getCommodity());	
		ps1.executeUpdate();
		ps1.close();

	}

	
	/**
	 * {@inheritDoc}
	 */	
	public Map getCommodityInfo(String commodity) throws Exception {
		
		PreparedStatement ps=this.connection().prepareStatement("Select * from commodity_info WHERE commodity=?");
		ps.setString(1, commodity);
		ResultSet rs=ps.executeQuery();
		Map record=null;
		while(rs.next()){			
			//record=new BasicRowProcessor().toMap(rs);
			record = GeneralUtilDB.resultsetToMap(rs);
		}
		return record;
	}
	
	
	/**
	 * {@inheritDoc}
	 */	
	public Map getContractInfo(String contract) throws Exception {
		
		PreparedStatement ps=this.connection().prepareStatement("Select * from contract_info WHERE ltrim(name)=?");
		ps.setString(1, contract.trim());
		ResultSet rs=ps.executeQuery();
		Map record=null;
		while(rs.next()){			
			//record=new BasicRowProcessor().toMap(rs);
			record = GeneralUtilDB.resultsetToMap(rs);
		}
		return record;
	}

	
	/**
	 * {@inheritDoc}
	 */	
	public Map getSecurityInfo(String security) throws Exception {
		
		PreparedStatement ps=this.connection().prepareStatement("Select * from security_info WHERE ltrim(name)=?");
		ps.setString(1, security.trim());
		ResultSet rs=ps.executeQuery();
		Map record=null;
		while(rs.next()){			
			//record=new BasicRowProcessor().toMap(rs);
			record = GeneralUtilDB.resultsetToMap(rs);
		}
		return record;
	}
	
	
	/**
	 * {@inheritDoc}
	 */	
	public Vector getContractBBInfo(String contract) throws Exception {
		
		//PreparedStatement ps=this.connection().prepareStatement("select bb_fieldname,value from contract_info_fields where contract_id in (select id from contract_info where name=?)");
		PreparedStatement ps=this.connection().prepareStatement("select b.friendly_name,value from contract_info_fields as a left outer join static_field_mapping as b on a.bb_fieldname=b.bb_fieldname	where a.contract_id in (select id from contract_info where name=?)");
		ps.setString(1, contract);		
		ResultSet rs=ps.executeQuery();
		Vector rtn=new Vector();
		while(rs.next()){
			//Map record=new BasicRowProcessor().toMap(rs);			
			Map record = GeneralUtilDB.resultsetToMap(rs);
			rtn.add(record);
			
		}
		return rtn;
	}
	

	/**
	 * {@inheritDoc}
	 */	
	public Vector getSecurityBBInfo(String security) throws Exception {
		
		//PreparedStatement ps=this.connection().prepareStatement("select bb_fieldname,value from contract_info_fields where contract_id in (select id from contract_info where name=?)");
		PreparedStatement ps=this.connection().prepareStatement("select b.friendly_name,value,b.bb_fieldname from security_info_fields as a left outer join security_static_fieldmapping as b on a.bb_fieldname=b.bb_fieldname	where a.security_id in (select id from security_info where name=?)");
		ps.setString(1, security);		
		ResultSet rs=ps.executeQuery();
		Vector rtn=new Vector();
		while(rs.next()){
			//Map record=new BasicRowProcessor().toMap(rs);	
			Map record = GeneralUtilDB.resultsetToMap(rs);
			rtn.add(record);
			
		}
		return rtn;
	}
	

	/**
	 * {@inheritDoc}
	 */	
	public Map getAllSecFieldValues(String friendlyname) throws Exception {
		
		log.debug("getAllSecFieldValue() called");
		PreparedStatement ps=this.connection().prepareStatement("select a.name,b.value from security_info as a left outer join security_info_fields as b on a.id=b.security_id left outer join security_static_fieldmapping as c on c.bb_fieldname=b.bb_fieldname where c.friendly_name=?");
		ps.setString(1, friendlyname);
		ResultSet rs=ps.executeQuery();
		LinkedHashMap<String,String> rtn=new LinkedHashMap();		
		while(rs.next()){			 
			String sec=rs.getString("name");
			//String fname=rs.getString("friendly_name");
			String fval=rs.getString("value");
			rtn.put(sec, fval);
			/*
			if(fname!=null  ){
				Map<String,String> record=null;
				if(rtn.containsKey(sec)){
					record=rtn.get(sec);
				}else{
					record=new LinkedHashMap();
					rtn.put(sec, record);
				}
				record.put(fname, fval);
			}
			*/
		}
		//log.debug("record:"+rtn);
		return rtn;
		
	}

}
