/******************************************************************************
*
* Copyright: Intellectual Property of Four Elements Capital Pte Ltd, Singapore.
* All rights reserved.
*
******************************************************************************/

package com.fourelementscapital.db.mariadb;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.Statement;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.StringTokenizer;
import java.util.TreeMap;
import java.util.Vector;
import java.util.regex.Pattern;

import org.apache.commons.dbutils.BasicRowProcessor;
import org.apache.logging.log4j.Logger;
import org.apache.logging.log4j.LogManager;

import com.fourelementscapital.db.GeneralUtilDB;
import com.fourelementscapital.db.UtilDB;
import com.fourelementscapital.db.vo.ValueObject;


/**
 * Util MySQL DAO implementation
 */
public class UtilDBMariaDB extends UtilDB{
	private Logger log=LogManager.getLogger(UtilDB.class.getName());
	
	
	/**
	 * {@inheritDoc}
	 */	
	public List listCommodities(Connection con) throws Exception {
		
		Statement st=con.createStatement();
		String query="SELECT COUNT(*) FROM information_schema.tables WHERE table_schema = '"+getDb()+"' AND table_name = 'asset_master'";
		
		ResultSet rs=st.executeQuery(query);
		Vector rtn=new Vector();
		if(rs.next()){
			Statement st1=con.createStatement();
			ResultSet rs1=st1.executeQuery("Select distinct commodity from asset_master ORDER BY commodity");
			while(rs1.next()){ 
				rtn.add(rs1.getString("commodity"));
			}
			st1.close();
			rs1.close();
		}
		rs.close();
		st.close();
		return rtn;
	}
	
	 
	/**
	 * {@inheritDoc}
	 */	
	public List listCommoditiesWithFilter(Connection con,String filter) throws Exception {
		
		Statement st=con.createStatement();
		
		String query="SELECT COUNT(*) FROM information_schema.tables WHERE table_schema = '"+getDb()+"' AND table_name = 'asset_master'";
		ResultSet rs=st.executeQuery(query);
		
		Vector rtn=new Vector();
		if(rs.next()){
			Statement st1=con.createStatement();
			ResultSet rs1=st1.executeQuery("Select distinct commodity from asset_master where SUBSTRING (commodity , 1, 1) in ("+filter+") ORDER BY commodity");
			while(rs1.next()){ 
				rtn.add(rs1.getString("commodity"));
			}
			st1.close();
			rs1.close();
		}
		rs.close();
		st.close();
		return rtn;
	}
	
	
	/**
	 * {@inheritDoc}
	 */	
	public List listCommoditiesFirstLetters(Connection con) throws Exception {
		
		Statement st=con.createStatement();
		
		//ResultSet rs=st.executeQuery("select * from sysobjects where id = object_id('asset_master') and OBJECTPROPERTY(id, 'IsUserTable') = 1");
		String query="SELECT COUNT(*) FROM information_schema.tables WHERE table_schema = '"+getDb()+"' AND table_name = 'asset_master'";
		ResultSet rs=st.executeQuery(query);
		
		Vector rtn=new Vector();
		if(rs.next()){
			Statement st1=con.createStatement();
			ResultSet rs1=st1.executeQuery("select distinct(SUBSTRING(UPPER(commodity) , 1, 1)) as l  from asset_master ORDER BY l");
			//ResultSet rs1=st1.executeQuery("select top 15 * from  (select distinct(SUBSTRING(commodity , 1, 1)) as l  from asset_master) as a");
			while(rs1.next()){ 
				rtn.add(rs1.getString("l"));
			}
			st1.close();
			rs1.close();
		}
		rs.close();
		st.close();
		return rtn;
	}
	
	
	/**
	 * {@inheritDoc}
	 */	
	public List uniqueFields(Connection con) throws Exception {
		
		    Vector rtn=new Vector();
			
			
			Statement st=con.createStatement();
			String query="SELECT COUNT(*) FROM information_schema.tables WHERE table_schema = '"+getDb()+"' AND table_name = 'asset_master'";
			ResultSet rs=st.executeQuery(query);
			if(rs.next()){
				Statement st1=con.createStatement();
				ResultSet rs1=st1.executeQuery("select distinct commodity_fieldoriginal as field from asset_master");
				while(rs1.next()){ 
					rtn.add(rs1.getString("field"));
				}
				st1.close();
				rs1.close();
			}
			st.close();
			rs.close();
			return rtn;
	}
	
	
	/**
	 * {@inheritDoc}
	 */	
	public List<Map> listAllTables4Fields(Connection con, String field_arr, String commodity_arr ) throws Exception {
		
	    Vector<Map> rtn=new Vector<Map>();
		Statement st1=con.createStatement();
		ResultSet rs1=st1.executeQuery("select * from asset_master where commodity_fieldoriginal IN ("+field_arr+") AND commodity IN ("+commodity_arr+")");
		
		while(rs1.next()){ 
			rtn.add(new BasicRowProcessor().toMap(rs1));
		}
		st1.close();
		rs1.close();
		return rtn;
	}
	
	
	/**
	 * {@inheritDoc}
	 */	
	public List<Map> listAllOrphanedAssets(Connection con ) throws Exception {
		
	    Vector<Map> rtn=new Vector<Map>();
		Statement st1=con.createStatement();
		//ResultSet rs1=st1.executeQuery("select * from asset_master where commodity_field not in(select name from sysobjects where name in (select commodity_field from asset_master) AND xtype='U')");
		ResultSet rs1=st1.executeQuery("select * from asset_master where commodity_field not in(select table_name from information_schema.tables where table_name in (select commodity_field from asset_master) and table_schema = '"+getDb()+"')");
		 
		while(rs1.next()){ 
			rtn.add(new BasicRowProcessor().toMap(rs1));
		}
		st1.close();
		rs1.close();
		return rtn;
	}
	
	
	/**
	 * {@inheritDoc}
	 */	
	public void removeAllOrphanedAssets(Connection con) throws Exception {
		
		 Vector<Map> rtn=new Vector<Map>();
			Statement st1=con.createStatement();
			st1.executeUpdate("delete from asset_master WHERE commodity_field IN (select commodity_field from asset_master where commodity_field not in(select table_name from information_schema.tables where table_name in (select commodity_field from asset_master) and table_schema = '"+getDb()+"'))");
			st1.close();
	}
	
	
	/**
	 * {@inheritDoc}
	 */	
	public List<String> listAllTables4Commodities(Connection con, String fieldtype,Vector<String> commodities) throws Exception {
		
	    Vector<String> rtn=new Vector<String>();
	    PreparedStatement ps=con.prepareStatement("select * from asset_master WHERE commodity_fieldoriginal=(select commodity_fieldoriginal from asset_master WHERE commodity_field=? LIMIT 1) AND commodity=?");
		
		for(String comms:commodities){
			ps.setString(1, fieldtype);
			ps.setString(2, comms);
			ResultSet rs1=ps.executeQuery();	    				
			while(rs1.next()){ 
				//String com=rs1.getString("commodity");				
				rtn.add(rs1.getString("commodity_field"));
			}			
			rs1.close();
		}
		ps.close();
		return rtn;
		

		/*
		 Vector<String> rtn=new Vector<String>();
	    	Statement st1=con.createStatement();
			ResultSet rs1=st1.executeQuery("select * from asset_master	 where commodity_fieldoriginal=(select top 1 commodity_fieldoriginal from asset_master  where commodity_field='"+fieldtype+"')");
			while(rs1.next()){ 
				String com=rs1.getString("commodity");
				if(commodities.contains(com)){
					rtn.add(rs1.getString("commodity_field"));
				}
			}
			st1.close();
			rs1.close();
			return rtn;
		*/
	
	}


	/**
	 * {@inheritDoc}
	 */	
	public String getFieldOriginal(Connection con,String tablename) throws Exception {
		
	    Vector<String> rtn=new Vector<String>();
	    PreparedStatement ps=con.prepareStatement("select commodity_fieldoriginal from asset_master WHERE commodity_field=?");
		ps.setString(1, tablename);
		ResultSet rs1=ps.executeQuery();
		String fname=null;
		if(rs1.next()){ 
				//String com=rs1.getString("commodity");				
			fname=rs1.getString("commodity_fieldoriginal");
		}			
			rs1.close();
		
		ps.close();
		return fname;
		

		/*
		 Vector<String> rtn=new Vector<String>();
	    	Statement st1=con.createStatement();
			ResultSet rs1=st1.executeQuery("select * from asset_master	 where commodity_fieldoriginal=(select top 1 commodity_fieldoriginal from asset_master  where commodity_field='"+fieldtype+"')");
			while(rs1.next()){ 
				String com=rs1.getString("commodity");
				if(commodities.contains(com)){
					rtn.add(rs1.getString("commodity_field"));
				}
			}
			st1.close();
			rs1.close();
			return rtn;
		*/
	
	}
	
	
	/**
	 * {@inheritDoc}
	 */	
	public Vector<ValueObject> listAllTables4Commodities2(Connection con, String fieldtype,Vector<String> commodities) throws Exception {
		
	    Vector<ValueObject> rtn=new Vector<ValueObject>();
	    PreparedStatement ps=con.prepareStatement("select * from asset_master WHERE commodity_fieldoriginal=(select commodity_fieldoriginal from asset_master  WHERE commodity_field=? LIMIT 1) AND commodity=?");
		for(String comms:commodities){
			ps.setString(1, fieldtype);
			ps.setString(2, comms);
			ResultSet rs1=ps.executeQuery();	    				
			while(rs1.next()){ 
				//String com=rs1.getString("commodity");		
				ValueObject vo=new ValueObject();
				vo.setKey(comms);
				vo.setValue(rs1.getString("commodity_field"));
				rtn.add(vo);
			}
			rs1.close();
		}
		ps.close();
		return rtn;
	}
	
	
	/**
	 * {@inheritDoc}
	 */	
	//public List<String> listAllContractTitles(Connection con, String jointables) throws Exception {
	public TreeMap<String,Vector> listAllContractTitles(Connection con, String jointables) throws Exception {
		
		TreeMap<String,Vector> rtn=new TreeMap<String,Vector>();
		Statement st1=con.createStatement();
		ResultSet rs1=st1.executeQuery("Select distinct contract,tbl_name FROM "+"("+jointables+") as outertable ORDER BY tbl_name");
		while(rs1.next()){
			String tbl=rs1.getString("tbl_name");
			Vector v;
			if(rtn.get(tbl)==null){
				v=new Vector();
				rtn.put(tbl,v);
			}else{
				v=rtn.get(tbl);
			}
			v.add(rs1.getString("contract"));
			//rtn.add(rs1.getString("contract"));
		}
		st1.close();
		rs1.close();
		return rtn;
	}
 
	
	/**
	 * {@inheritDoc}
	 */	
	//public List<String> listAllContractTitles(Connection con, String jointables) throws Exception {
	public TreeMap<String,Vector> listAllContractTitles2(Connection con, String jointables) throws Exception {
		
		TreeMap<String,Vector> rtn=new TreeMap<String,Vector>();
		Statement st1=con.createStatement();
		ResultSet rs1=st1.executeQuery("Select distinct contract,tbl_name FROM "+"("+jointables+") as outertable ORDER BY tbl_name");
		while(rs1.next()){
			String tbl=rs1.getString("tbl_name");
			Vector v;
			if(rtn.get(tbl)==null){
				v=new Vector();
				rtn.put(tbl,v);
			}else{
				v=rtn.get(tbl);
			}
			v.add(rs1.getString("contract"));
			//rtn.add(rs1.getString("contract"));
		}
		st1.close();
		rs1.close();
		return rtn;
	}
	

	/**
	 * {@inheritDoc}
	 */	
	public List<String> listAllContractTitles2LCase(Connection con, String jointables) throws Exception {
		
	    Vector<String> rtn=new Vector<String>();
		Statement st1=con.createStatement();
		ResultSet rs1=st1.executeQuery("Select distinct contract FROM "+"("+jointables+") as outertable");
		while(rs1.next()){ 
			rtn.add(rs1.getString("contract").toLowerCase());
		}
		st1.close();
		rs1.close();
		return rtn;
	}

	
	/**
	 * {@inheritDoc}
	 */	
	public boolean removeCommodityAndFields(Connection con,String commodity) throws Exception {
		 
			
			PreparedStatement ps=con.prepareStatement("SELECT commodity_field FROM  asset_master WHERE commodity=?");
			ps.setString(1, commodity);
			ResultSet rs=ps.executeQuery();
			while(rs.next()){
				
				String field=rs.getString("commodity_field");
				
				Statement st1=con.createStatement();
				st1.execute("DROP TABLE  "+field+"");
				st1.close();
				
				String query="DELETE FROM asset_master WHERE commodity_field='"+field+"'";
				Statement st=con.createStatement();
				st.execute(query);
				st.close();				
			}
			rs.close();
			ps.close();			
 			return true;
	}
	

	/**
	 * {@inheritDoc}
	 */	
	public Vector getRawData(String datequery,Connection con,String tablename, String commodity) throws Exception {		 
		datequery=GeneralUtilDB.convertFromSQL(datequery);
		PreparedStatement ps=con.prepareStatement("SELECT *,DATE_FORMAT(cdate,'%d/%m/%Y') as cdate1 FROM  "+tablename+" WHERE contract=? AND "+datequery+" ORDER BY CDATE DESC");
		ps.setString(1, commodity);
		ResultSet rs=ps.executeQuery();
		Vector rtn=new Vector();
		while(rs.next()){
			rtn.add(new BasicRowProcessor().toMap(rs));
		
		}
		rs.close();
		ps.close();			
		return rtn;	
	}
	

	/**
	 * {@inheritDoc}
	 */	
	public List getRecordCount4Contracts(Connection con, String commodity, String contract ) throws Exception {
		
		//String query="select 'select count(*),contract,'+char(39)+'cl'+char(39)+' from ['+commodity_field+'] group by contract' from asset_master where commodity='"+commodity+"' ";
		String query="select 'select '+char(39)+char(39)+' as rw,'+char(39)+char(39)+' as field,'+char(39)+char(39) union all select 'UNION ALL select count(*) as rw,'+char(39)+commodity_fieldoriginal+char(39)+' as field,'+char(39)+'"+commodity+"'+char(39)+' from '+commodity_field+' where contract='+char(39)+'"+contract+"'+char(39)+' group by contract ' from asset_master where commodity='"+commodity+"'";
		Statement st=con.createStatement();
		ResultSet rs=st.executeQuery(query);
		String query1="";
		while(rs.next()){
			query1+="  "+rs.getString(1);
		}
		//System.out.println("UtilDB:"+query);
		//System.out.println("UtilDB:"+query1);
		rs.close();
		st.close();

		Vector rtn=new Vector();
		if(!query1.equals("")){
		
			PreparedStatement ps1=con.prepareStatement(query1);			 
			ResultSet rs1=ps1.executeQuery();			 
			while(rs1.next()){
				rtn.add(new BasicRowProcessor().toMap(rs1));
			
			}
			rs1.close();
			ps1.close();			
		}
		return rtn;
		
		
	}


	/**
	 * {@inheritDoc}
	 */	
	public Map getRawData2Contract(String datequery,Connection con,String fieldname, String contrat,String commodity) throws Exception {
		datequery=GeneralUtilDB.convertFromSQL(datequery);
		log.debug("commodity:"+commodity+" fieldanme:"+fieldname);
		TreeMap t=new TreeMap();
		Vector rtn=new Vector();
		PreparedStatement ps0=con.prepareStatement("select * from asset_master WHERE commodity=? AND commodity_fieldoriginal=?");
		ps0.setString(1, commodity);
		ps0.setString(2, fieldname);
		ResultSet rs0=ps0.executeQuery();
		if(rs0.next()){
			String fieldname_t=rs0.getString("commodity_field");
			PreparedStatement ps=con.prepareStatement("SELECT DATE_FORMAT(cdate,'%d/%m/%Y') as cdate1,  * FROM  "+fieldname_t+" WHERE contract=? AND "+datequery+" ORDER BY CDATE DESC");
			log.debug("query:SELECT CONVERT  ( varchar(40),cdate,103 ) as cdate1,  * FROM  ["+fieldname_t+"] WHERE contract=? AND "+datequery+" ORDER BY CDATE DESC");
			ps.setString(1, contrat);
			ResultSet rs=ps.executeQuery();			
			
			while(rs.next()){
				rtn.add(new BasicRowProcessor().toMap(rs));
			}
			rs.close();
			ps.close();
			t.put("tablename",fieldname_t);
			t.put("data", rtn);
		}
		rs0.close();
		ps0.close();
		
		return t;	
	}
	
	
	/**
	 * {@inheritDoc}
	 */	
	public void deleteRawData(Connection con,String tablename,Map<String,String> data) throws Exception {
		 
		
		PreparedStatement ps=con.prepareStatement("DELETE FROM "+tablename+" WHERE cdate=? AND contract=?");
		SimpleDateFormat format=new SimpleDateFormat("dd/MM/yyyy");		
		java.sql.Date d1=new java.sql.Date(format.parse(data.get("cdate1")).getTime());
		ps.setDate(1,d1);		 
		ps.setString(2,data.get("contract"));
		ps.executeUpdate();
		ps.close();			
	}
	
	
	/**
	 * {@inheritDoc}
	 */	
	public void updateRawData(Connection con,String tablename,Map<String,String> olddata,Map<String,String> newdata) throws Exception {
		 
		
		String query="UPDATE "+tablename+" SET cdate=?, contract=?,val=? WHERE cdate=? AND contract=?";
		String sval=null;
		if(newdata.get("sval")!=null){
			sval=newdata.get("sval");
		}else{
			if(olddata.get("sval")!=null){
				sval=olddata.get("sval");
			}
		}
		if(sval!=null){
			query="UPDATE "+tablename+" SET cdate=?, contract=?,sval=? WHERE cdate=? AND contract=?";
		}
		
		PreparedStatement ps=con.prepareStatement(query);
		SimpleDateFormat format=new SimpleDateFormat("dd/MM/yyyy");
		
		java.sql.Date d0;
		if(newdata.get("cdate1")!=null){
			d0=new java.sql.Date(format.parse(newdata.get("cdate1")).getTime());
		}else{
			d0=new java.sql.Date(format.parse(olddata.get("cdate1")).getTime());
		}
		
		double val;
		if(newdata.get("val")!=null){
			val=Double.parseDouble(newdata.get("val"));
		}else{
			if(sval==null){
			  val=Double.parseDouble(olddata.get("val"));
			}
			val=0;
		}

		
		
		
		java.sql.Date d1=new java.sql.Date(format.parse(olddata.get("cdate1")).getTime());
		
		
		ps.setDate(1,d0);		
		ps.setString(2,(newdata.get("contract")!=null)?newdata.get("contract"):olddata.get("contract"));
		if(val!=0){
			ps.setDouble(3,val);
		}else{
			if(sval!=null){
				ps.setString(3,sval);
			}
		}
		
		ps.setDate(4,d1);		 
		ps.setString(5,olddata.get("contract"));
		ps.executeUpdate();
		ps.close();			
	}
	
	
	/**
	 * {@inheritDoc}
	 */	
	public boolean removeFieldTable(Connection con,String ftable) throws Exception {
		 
			String query="DELETE FROM asset_master WHERE commodity_field='"+ftable+"'";
			Statement st=con.createStatement();			
			st.execute(query);
			Statement st1=con.createStatement();
			st1.execute("DROP TABLE "+ftable);
			st1.close();
			st.close();
			return true;
	}
	
	
	/**
	 * {@inheritDoc}
	 */	
	public boolean renameFieldTable(Connection con,String ftable, String fieldoriginal) throws Exception {
		 
		//String fieldtable=HistoricalMarketData.resolvedColName(com_sec1)+"_"+HistoricalMarketData.resolvedColName(dbfieldname);
		
		PreparedStatement ps1=con.prepareStatement("SELECT * FROM asset_master WHERE commodity_field=?");
		ps1.setString(1, ftable);
		ResultSet rs1=ps1.executeQuery();
		String newtable=null;
		if(rs1.next()){			
			newtable=rs1.getString("commodity");
			newtable=resolvedColName(newtable)+"_"+resolvedColName(fieldoriginal);			
		}
		if(newtable!=null){
		
			Statement st=con.createStatement();
			//System.out.println("sp_rename '"+ftable+"','"+newtable+"'");
			st.executeUpdate("sp_rename '"+ftable+"','"+newtable+"'");
			
			st.close();
			
			//String query="UPDATE asset_master SET commodity_fieldoriginal=?  WHERE commodity_field=?";		
			//Statement st=con.createStatement();
			PreparedStatement ps=con.prepareStatement("UPDATE asset_master SET commodity_fieldoriginal=?,commodity_field=?  WHERE commodity_field=?");
			ps.setString(1, fieldoriginal);
			//ps.setString(2, ftable);
			ps.setString(2, newtable);
			ps.setString(3, ftable);
			ps.executeUpdate();		
			
			//st1.execute("DROP TABLE [dbo].["+ftable+"]");
			ps.close();
			return true;
		}else{
		   return false;	
		}
		
		//st.close();
		//return true;
}
	
	
	private String resolvedColName(String s) {
		String rtn = null;
		if (s != null && s.length() > 0) {
			s=s.trim();
			int maxLeng = 25;
			Pattern escaper = Pattern.compile("([^a-zA-z0-9])");
			s = s.replaceAll(" ", "_").toLowerCase();
			s = escaper.matcher(s).replaceAll("");
			s = s.substring(0, (s.length() > maxLeng) ? maxLeng : s.length());
			rtn = s.replaceAll(" ", "_").toLowerCase();
		}
		return rtn;
	}	
	
	
	/**
	 * {@inheritDoc}
	 */	
	public List listFieldTables4Admin(Connection con,String mtable) throws Exception {
	
			Statement st1=con.createStatement();
			ResultSet rs1=st1.executeQuery("Select commodity_field,commodity_fieldoriginal from asset_master WHERE commodity='"+mtable+"'");
			String query="";
			int count=0;
			while(rs1.next()){
				if(count>0){				
					query+=" UNION ALL ";
				}
				count++;
				query+="SELECT count(*) as recs,'"+rs1.getString("commodity_field")+"' as tabl,'"+rs1.getString("commodity_fieldoriginal")+"' as title FROM "+rs1.getString("commodity_field");
			}
			st1.close();
			rs1.close();
			Vector rtn=new Vector();
			if(count>0){
				Statement st=con.createStatement();			
				
				ResultSet rs=st.executeQuery(query);
				
				while(rs.next()){
					TreeMap t=new TreeMap();
					t.put("total", rs.getObject("recs"));
					t.put("ftable", rs.getObject("tabl"));
					t.put("fname", rs.getObject("title"));
					t.put("commodity", mtable);
					rtn.add(t);
				}
				st.close();
				rs.close();
			}
			return rtn;
	}
	
	
	/**
	 * {@inheritDoc}
	 */	
	public List listFieldTables(Connection con,String mtable) throws Exception {
		
		Statement st=con.createStatement();
		
		
				
		
		//ResultSet rs=st.executeQuery("select * from sysobjects where id = object_id('asset_master') and OBJECTPROPERTY(id, 'IsUserTable') = 1");
		ResultSet rs=st.executeQuery("select table_name from information_schema.tables where table_name 'asset_master' and table_schema = '"+getDb()+"')");
		Vector rtn=new Vector();
		if(rs.next()){
			PreparedStatement st1=con.prepareStatement("Select * from asset_master where commodity=?");
			st1.setString(1,mtable);
			ResultSet rs1=st1.executeQuery();
			while(rs1.next()){ 
				TreeMap rec=new TreeMap();
				rec.put("fieldname",rs1.getString("commodity_fieldoriginal"));
				rec.put("fieldtable",rs1.getString("commodity_field"));
				rtn.add(rec);
			}
			st1.close();
			rs1.close();
		}
		rs.close();
		st.close();
		return rtn;
	}

	
	/**
	 * {@inheritDoc}
	 */	
	public List  listAllUniqueFields(Connection con) throws Exception {
		
		Statement st=con.createStatement();
		
		//ResultSet rs=st.executeQuery("select * from sysobjects where id = object_id('asset_master') and OBJECTPROPERTY(id, 'IsUserTable') = 1");
		ResultSet rs=st.executeQuery("select table_name from information_schema.tables where table_name='asset_master' and table_schema = '"+getDb()+"'");
		Vector rtn=new Vector();
		if(rs.next()){
			Statement st1=con.createStatement();
		 
			ResultSet rs1=st1.executeQuery("select commodity_fieldoriginal,MAX(commodity),MAX(commodity_field) as commodity_field1 from asset_master group by commodity_fieldoriginal");
			while(rs1.next()){ 
				TreeMap rec=new TreeMap();
				rec.put("fieldname",rs1.getString("commodity_fieldoriginal"));
				rec.put("fieldtable",rs1.getString("commodity_field1"));
				rtn.add(rec);
			}
			st1.close();
			rs1.close();
		}
		rs.close();
		st.close();
		 
		return rtn;
		
		
		
	}

	
	
	
	/**
	 * {@inheritDoc}
	 */	
	public void deleteContracts(Connection con, String field_tablename, String contract) throws Exception {
		PreparedStatement ps=con.prepareStatement("delete  from  "+field_tablename+"  WHERE  contract=?");
		ps.setString(1, contract);
		ps.executeUpdate();
		ps.close();
	}
	
	
	/**
	 * {@inheritDoc}
	 */	
	public Vector getUniqueContracts(Connection con, String field_tablename) throws Exception {
		//PreparedStatement ps=con.prepareStatement("select distinct contract from ["+field_tablename+"]");
		//ps.setString(1, field_tablename);
		PreparedStatement ps=con.prepareStatement("select * from (select contract,count(*) recs,min(cdate) as first,max(cdate) as last  from   "+field_tablename+" group by contract) as outtable order by last desc");
		
		ResultSet rs=ps.executeQuery();
		Vector result=new Vector();
		
		while(rs.next()){
			//result.add(rs.getString("contract"));
			Map mp=new BasicRowProcessor().toMap(rs);
			Date s1=(Date)mp.get("first");
			Date s2=(Date)mp.get("last");
			if(s1!=null && s2!=null){
				String dt=new SimpleDateFormat("dd MMM yyyy").format(s1)+" - "+new SimpleDateFormat("dd MMM yyyy").format(s2);
				mp.put("dates",dt);
			}
			
			result.add(mp);
		}
		rs.close();
		ps.close();
		return result;
	}
	
	
	/**
	 * {@inheritDoc}
	 */	
	public Map listAllCommoditiesAndFields(Connection con) throws Exception {
		TreeMap<String,Vector> rtn=new TreeMap<String,Vector>();
		
		Statement st1=con.createStatement();
		//ResultSet rs1=st1.executeQuery("select * from sysobjects where id = object_id('asset_master') and OBJECTPROPERTY(id, 'IsUserTable') = 1");
		ResultSet rs1=st1.executeQuery("select table_name from information_schema.tables where table_name='asset_master' and table_schema = '"+getDb()+"'");
		
		if(rs1.next()){
			Statement st=con.createStatement();		
			ResultSet rs=st.executeQuery("select * from asset_master order by commodity");
			//String oldcommodity=null;
			
			while(rs.next()){
				String commodity=rs.getString("commodity");
				Vector<ValueObject> records;
				if(rtn.get(commodity)==null){
					records=new Vector<ValueObject>();
					rtn.put(commodity,records);
				}else{
					records=rtn.get(commodity);
				}
				ValueObject vo=new ValueObject();
				vo.setKey(rs.getString("commodity_field"));
				vo.setValue(rs.getString("commodity_fieldoriginal"));
				records.add(vo);
			}
			
		}
		st1.close();
		rs1.close();
		return rtn;
	}
	
	
	/**
	 * {@inheritDoc}
	 */	
	public void LastSyncUpdate(Connection con) throws Exception {
		
		//String query="IF NOT EXISTS (select * from sysobjects where id = object_id('asset_sync') and OBJECTPROPERTY(id, 'IsUserTable') = 1) ";
		//query+="BEGIN ";
		String query="CREATE TABLE IF NOT EXISTS asset_sync ( ";
		query+="syncdate datetime";
		query+=") ";
		
		 
		 
		Statement st=con.createStatement();
		st.execute(query);
		st.close();
		
		Statement st2=con.createStatement(); //.prepareStatement("INSERT into asset_sync(syncdate) VALUES(getdate()) ");
		//st2.setDate(1, new java.sql.Date(date.getTime()));
		st2.executeUpdate("INSERT into asset_sync(syncdate) VALUES(getdate()) ");
		st2.close();
	}
	
	
	/**
	 * {@inheritDoc}
	 */	
	public int lastSyncDays(Connection con,Date today) throws Exception {
		int rtn=-1;
		Statement st=con.createStatement();
	 	//ResultSet rs=st.executeQuery("select * from sysobjects where id = object_id('asset_sync') and OBJECTPROPERTY(id, 'IsUserTable') = 1 ");
		ResultSet rs=st.executeQuery("select table_name from information_schema.tables where table_name 'asset_sync' and table_schema = '"+getDb()+"')");
		
	 	if(rs.next()){
	 		Statement st1=con.createStatement();
	 		ResultSet rs1=st1.executeQuery("select top 1 syncdate from asset_sync ORDER BY syncdate DESC");
	 		if(rs1.next()){
	 			java.sql.Date sdate=rs1.getDate("syncdate");
	 			Calendar tday=Calendar.getInstance();
	 			tday.setTime(today);
	 			Calendar sday=Calendar.getInstance();
	 			sday.setTime(sdate);
	 			rtn=daysBetween(sday, tday);
	 		}
	 	}
	 	rs.close();
		st.close();		
		return rtn;			
	}

	
	/**
	 * {@inheritDoc}
	 */	
	public Date lastSyncDate(Connection con) throws Exception {
		Date rtn=null;
		Statement st=con.createStatement();
	 	//ResultSet rs=st.executeQuery("select * from sysobjects where id = object_id('asset_sync') and OBJECTPROPERTY(id, 'IsUserTable') = 1 ");
		ResultSet rs=st.executeQuery("select table_name from information_schema.tables where table_name 'asset_sync' and table_schema = '"+getDb()+"')");
		
	 	if(rs.next()){
	 		Statement st1=con.createStatement();
	 		ResultSet rs1=st1.executeQuery("select top 1 syncdate from asset_sync ORDER BY syncdate DESC");
	 		if(rs1.next()){
	 			rtn=new SimpleDateFormat("yyyy-MM-dd HH:mm:ss.SSS").parse(rs1.getString("syncdate")); 			
	 		}
	 	}
	 	rs.close();
		st.close();		
		return rtn;		
	}

	
	/**
	 * Calculate days between. Using Calendar - THE CORRECT WAY
	 * @param startDate start date
	 * @param endDate end date
	 * @return days
	 */		
	  private int daysBetween(Calendar startDate, Calendar endDate) {  
	    Calendar date = (Calendar) startDate.clone();  
	    int daysBetween = 0;  
	    while (date.before(endDate)) {  
	      date.add(Calendar.DAY_OF_MONTH, 1);  
	      daysBetween++;  
	    }  
	    return daysBetween;  
	  }
	  
  
	/**
	 * {@inheritDoc}
	 */	
	  public TreeMap dailyStrategyData(Connection con,String date, String contractname, String fields) throws Exception {

		  String q;	
		  
		  StringTokenizer stt=new StringTokenizer(fields,"|");
		  String fields1="";
		  while(stt.hasMoreTokens()){
			  String token=stt.nextToken();
			  fields1+=fields1.equals("")? "'"+token+"'":",'"+token+"'";
		  }
		  
		  if(!fields1.equals("")){
			  fields1=" WHERE commodity_fieldoriginal IN ("+fields1+") ";
		  }
		  log.debug("fields1:"+fields1);
		  
		  if(date!=null){
			  q="select 'select contract,val,sval,'+CHAR(39)+commodity+CHAR(39)+' as commodity,'+CHAR(39)+commodity_fieldoriginal+CHAR(39)+' as fieldname from '+commodity_field+' WHERE cdate='+CHAR(39)+'"+date+"'+CHAR(39)+' AND contract='+CHAR(39)+'"+contractname+"'+CHAR(39)+' union ' as q from asset_master "+fields1;
		  }else{
			  q="select 'select top 1 contract,val,sval,'+CHAR(39)+commodity+CHAR(39)+' as commodity,'+CHAR(39)+commodity_fieldoriginal+CHAR(39)+' as fieldname from '+commodity_field+' WHERE cdate=(select max(cdate) from '+commodity_field+') AND contract='+CHAR(39)+'"+contractname+"'+CHAR(39)+' union ' as q from asset_master "+fields1;
			  date="";
		  }
		  log.debug(q);
		  Statement st=con.createStatement();
		  ResultSet rs=st.executeQuery(q);
		  String qr="";
		  while(rs.next()){
			  qr+=rs.getString("q");
			  /*
			  rs.getDate("cdate");
			  rs.getString("contract");
			  rs.getDouble("val");
			  rs.getString("commodity");
			  rs.getString("fieldname");
			  rs.getString("fieldcode");
			  */
		 }
		 if(!qr.equals("")){
			 qr="select * from ("+qr+" select null,null,null,null,null) as t  where commodity is not null order by contract,commodity"; 
		 }
		 rs.close();
		 st.close();
		 
		 System.out.println("UtilDB.dailyStrategyData():qr:"+qr);
		 
		 TreeMap rows=new TreeMap();
		 if(!qr.equals("")){
			 Statement st1=con.createStatement();
			 ResultSet rs1=st1.executeQuery(qr);
			 while(rs1.next()){
				  String field=rs1.getString("fieldname");
				  String ky=rs1.getString("commodity")+"|"+rs1.getString("contract");


				  TreeMap data;
				  if(rows.get(field)!=null){
					  data=(TreeMap)rows.get(field);
				  }else{
					  data=new TreeMap();
					  rows.put(field,data);
				  }
				  
				  if(rs1.getObject("sval")!=null){
					  data.put(ky,rs1.getObject("sval"));
				  }else{
					  data.put(ky,rs1.getDouble("val"));
				  }
				 // rs.getString("fieldname");			  				 
				 
			 }
			 st1.close();
			 rs1.close();
		 }
		 return rows;
	  }
	  
	  
	/**
	 * {@inheritDoc}
	 */	
	  public TreeMap listSavedChart(Connection con) throws Exception {
			
		    TreeMap rtn=new TreeMap();
			
			Statement st=con.createStatement();
			//ResultSet rs=st.executeQuery("select * from sysobjects where id = object_id('saved_charts') and OBJECTPROPERTY(id, 'IsUserTable') = 1");
			ResultSet rs=st.executeQuery("select table_name from information_schema.tables where table_name 'saved_charts' and table_schema = '"+getDb()+"')");
			if(rs.next()){
				Statement st1=con.createStatement();
				ResultSet rs1=st1.executeQuery("select id,chartname from saved_charts");
				while(rs1.next()){ 
					rtn.put(rs1.getInt("id"),rs1.getString("chartname"));
				}
				st1.close();
				rs1.close();
			}
			st.close();
			rs.close();
			return rtn;
	 }

	  
	/**
	 * {@inheritDoc}
	 */	
	  public void deleteSavedChart(Connection con, int id) throws Exception {
		  	PreparedStatement st1=con.prepareStatement("DELETE FROM saved_charts WHERE id=?" );
		  	st1.setInt(1, id);
		  	st1.executeUpdate();
			st1.close();		  
	  }
	  
  
	/**
	 * {@inheritDoc}
	 */	  
	  public Map getSavedChartItem(Connection con, int id) throws Exception {
		  	String query1="SELECT * FROM saved_charts WHERE id=?";		  	
		  	PreparedStatement ps=con.prepareStatement(query1);
		  	ps.setInt(1, id);
		  	ResultSet rs=ps.executeQuery();
		  	TreeMap data=null;
		  	if(rs.next()){
		  		data=new TreeMap();
		  	
		  		data.put("chartname",rs.getString("chartname"));		 
				data.put("datestring",rs.getString("datestring"));
				data.put("source1",rs.getString("source1"));
				data.put("security1",rs.getString("security1"));
				data.put("field1",rs.getString("field1"));
				data.put("source2",rs.getString("source2"));
				data.put("security2",rs.getString("security2"));
				data.put("field2",rs.getString("field2"));
				data.put("commodity1",rs.getString("commodity1"));
				data.put("commodity2",rs.getString("commodity2"));
		  	}
			rs.close();		
			ps.close();
			return data;
	  }
	  
	  
	/**
	 * {@inheritDoc}
	 */	
	  public void savedChart(Connection con,Map data) throws Exception {
			
		  
		  	///String query="IF NOT EXISTS (select * from sysobjects where id = object_id('saved_charts') and OBJECTPROPERTY(id, 'IsUserTable') = 1) ";
			//query+="BEGIN ";
			String query="CREATE TABLE IF NOT EXISTS saved_charts ( ";
			query+="id int IDENTITY, chartname varchar(250),datestring  varchar(250),source1 varchar(50),security1 text,field1 varchar(250),source2 varchar(50),security2 text,field2 varchar(250),commodity1 varchar(250), commodity2 varchar(250)";
			query+=") ";
			//query+="END ";
			
			Statement st=con.createStatement();
			st.execute(query);
			st.close();
			
			String query1="INSERT INTO saved_charts(chartname,datestring,source1,security1,field1,source2,security2,field2,commodity1,commodity2) VALUES(?,?,?,?,?,?,?,?,?,?)";
			PreparedStatement ps=con.prepareStatement(query1);
			ps.setString(1, (String)data.get("chartname"));			
			ps.setString(2, (String)data.get("datestring"));
			ps.setString(3, (String)data.get("source1"));
			ps.setString(4, (String)data.get("security1"));
			ps.setString(5, (String)data.get("field1"));
			ps.setString(6, (String)data.get("source2"));
			ps.setString(7, (String)data.get("security2"));
			ps.setString(8, (String)data.get("field2"));
			ps.setString(9, (String)data.get("commodity1"));
			ps.setString(10, (String)data.get("commodity2"));
			ps.executeUpdate();
					
			ps.close();
	 }




	/**
	 * {@inheritDoc}
	 */	
	@Override
	public Map getRawData3(String datequery, Connection con, String tablename,
			String commodity) throws Exception {
		log.debug("commodity:"+commodity+" tablename:"+tablename);
		TreeMap t=new TreeMap();
		Vector rtn=new Vector();
		PreparedStatement ps0=con.prepareStatement("select * from asset_master WHERE commodity=? AND commodity_field=?");
		ps0.setString(1, commodity);
		ps0.setString(2, tablename);
		ResultSet rs0=ps0.executeQuery();
		if(rs0.next()){
			//String fieldname_t=rs0.getString("commodity_field");
			PreparedStatement ps=con.prepareStatement("SELECT CONVERT  ( varchar(40),cdate,103 ) as cdate1,  * FROM  ["+tablename+"] WHERE contract=? AND "+datequery+" ORDER BY CDATE DESC");
			//log.debug("query:SELECT CONVERT  ( varchar(40),cdate,103 ) as cdate1,  * FROM  ["+fieldname_t+"] WHERE contract=? AND "+datequery+" ORDER BY CDATE DESC");
			ps.setString(1, commodity);
			ResultSet rs=ps.executeQuery();			
			
			while(rs.next()){
				rtn.add(new BasicRowProcessor().toMap(rs));
			}
			rs.close();
			ps.close();
			t.put("tablename",tablename);
			t.put("data", rtn);
		}
		rs0.close();
		ps0.close();
		
		
		return t;	
	}


	/**
	 * {@inheritDoc}
	 */	
	@Override
	public List listAllUniqueFields(Connection con, String commodity)
			throws Exception {
		// TODO Auto-generated method stub
		return null;
	}



	/**
	 * {@inheritDoc}
	 */	
	@Override
	public ArrayList showOldConnections(int minutesbefore, String computername)
			throws Exception {
		// TODO Auto-generated method stub
		return null;
	}



	/**
	 * {@inheritDoc}
	 */	
	@Override
	public void killConnections(List<Integer> conids) throws Exception {
		// TODO Auto-generated method stub
		
	}
	
}
