/******************************************************************************
*
* Copyright: Intellectual Property of Four Elements Capital Pte Ltd, Singapore.
* All rights reserved.
*
******************************************************************************/

package com.fourelementscapital.fileutils;

import java.io.BufferedReader;
import java.io.File;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.Map;

/**
 * Utility class to find specific string in files. Only works in Unix environment.
 */
public class FindStringInFiles {
	
	/**
	 * Private constructor
	 */	
	private FindStringInFiles(){
	}
	
	/**
	 * Search text
	 * @param word Text to search
	 * @param folder Folder where files contain string are located
	 * @param include_ext Include file extension of the file
	 * @return search result in Map
	 * @throws Exception
	 */		
	public static Map search(String word, String folder, String include_ext) throws Exception {
		
		   Process process=null;
		   Runtime runtime = Runtime.getRuntime();
		 	 
		   String command[]={"grep","-i","-n","-r","--exclude=*.svn-base","--include=*"+include_ext,word, folder};
		   process = runtime.exec(command);		   
		   BufferedReader br2 = new BufferedReader(new InputStreamReader(process.getInputStream()));
		   
		   LinkedHashMap<String,ArrayList> rtn=new LinkedHashMap();
		   
		   String line;
		   String content="";
		   while ((line = br2.readLine()) != null)   {
			   
			    if(line!=null && !line.trim().equals("") && line.contains(":") && line.split(":").length>2 ){
			    	 String[] arg=line.split(":");
			    	 String f,ln;
			    	 String occu=""; 
			    	 
			    	//windows
			    	if(System.getProperty("os.name").toLowerCase().equals("freebsd")){
			    		f=arg[0];		    	
				    	ln=arg[1];
				    	if(arg.length>=3){
				    		for(int i=2;i<arg.length;i++)	occu+=" "+arg[i];			    		
				    	}
				    	
			    	}else{
			    		//unix
			    		f=arg[0]+arg[1];		    	
			    		ln=arg[2];
				    	if(arg.length>=4){
				    		for(int i=3;i<arg.length;i++)	occu+=" "+arg[i];			    		
				    	}
				    
			    	}
			    	
			    	String filename=new File(f).getName();			    	
			    	if(rtn.get(filename)==null)		rtn.put(filename, new ArrayList());
			    	rtn.get(filename).add(ln+"|"+occu);
			    }
		   }

		   process.destroy();
		   
		   return rtn;
	}	

}


 