/******************************************************************************
*
* Copyright: Intellectual Property of Four Elements Capital Pte Ltd, Singapore.
* All rights reserved.
*
******************************************************************************/

package com.fourelementscapital.fileutils;

import java.util.Random;

/**
 * Generate random string by specify string length.
 */
public class RandomString {
	
	  private static final char[] symbols = new char[36];

	  static {
	    for (int idx = 0; idx < 10; ++idx)
	      symbols[idx] = (char) ('0' + idx);
	    for (int idx = 10; idx < 36; ++idx)
	      symbols[idx] = (char) ('a' + idx - 10);
	  }

	  private static char[] buffer;
	  private static Random rand = new Random();
	  
	  /**
	   * Private constructor
	   */		  
	  private RandomString() {
	  }	  
	  
	  
	/**
	 * Generate random string with length of the string as input parameter 
	 * @param length Length of the generated string
	 * @return random string
	 */		  
	  public static String getString(int length) {
		    if (length < 1) {
			      throw new IllegalArgumentException("length < 1: " + length);
		    }
		    buffer = new char[length];	
		    for (int idx = 0; idx < buffer.length; ++idx) {
		    	buffer[idx] = symbols[rand.nextInt(symbols.length)];
		    }
		    return new String(buffer);		    
	  }

}


 