/******************************************************************************
*
* Copyright: Intellectual Property of Four Elements Capital Pte Ltd, Singapore.
* All rights reserved.
*
******************************************************************************/

package com.fourelementscapital.fileutils;

import java.util.ArrayList;
import java.util.List;
import java.util.StringTokenizer;

/**
 * Split string into List by checking separator in it
 */
public class SplitString {

	/**
	 * Private constructor
	 */	
	private SplitString(){
	}
	
	/**
	 * Get separator character. Check ',' or '\r\n'
	 * @param separator Separator char 
	 * @return separator
	 */			
	private static String getSeparatorChar(String str){
		return (str.indexOf(",")>=0)?",":"\r\n";
	}
	
	/**
	 * Split string into List
	 * @param str String to be split
	 * @return split string in list
	 */		
	public static List<String> split(String str){
		
		List<String> rtn=new ArrayList<String>();
		if(str!=null && !str.equals("")){
			String pline=getSeparatorChar(str);
			StringTokenizer st=new StringTokenizer(str,pline);
			while(st.hasMoreTokens()){
				rtn.add(st.nextToken());
			}
		}
		return rtn;
	}
	
	/**
	 * Split string then convert to lower case
	 * @param str String to be split
	 * @return split string in list
	 */			
	public static List<String> splitLowerCase(String str){
		
		List<String> rtn=new ArrayList<String>();
		if(str!=null && !str.equals("")){
			String pline=getSeparatorChar(str);
			StringTokenizer st=new StringTokenizer(str,pline);
			while(st.hasMoreTokens()){
				rtn.add(st.nextToken().toLowerCase());
			}
		}
		return rtn;
	}
	
	/**
	 * Split string then trim
	 * @param str String to be split
	 * @return split string in list
	 */		
	public static List<String> splitTrim(String str){
		
		List<String> rtn = new ArrayList<String>();
		if(str!=null && !str.equals("")){
			String pline=getSeparatorChar(str);
			StringTokenizer st=new StringTokenizer(str,pline);
			while(st.hasMoreTokens()){
				rtn.add(st.nextToken().trim());
			}
		}
		return rtn;
	}	
}

 
